classdef attentionLayer < nnet.layer.Layer 
    % custom attention layer.

    properties
        NumHeads;
    end
    
    methods
        function layer = attentionLayer(NumInputs,NumHeads,Name) 
            % layer = attentionLayer(args) creates a Transformer encoder layer
            % with numChannels channels.
            %
            % layer = attentionLayer(Name=name) also specifies the
            % layer name.

    
            % Set layer name.
            layer.Name = Name;

            % Set layer description.
            layer.Description = "Attention layer";
            
            % Set layer type.
            layer.Type = "Attention";

            layer.NumInputs = NumInputs;
            layer.NumHeads = NumHeads;

        end
        
        function Z = predict(layer, varargin)
            % Forward input data through the layer at prediction time and
            % output the result.
            %
            % Inputs:
            %         layer - Layer to forward propagate through 
            %         X     - Input data
            % Output:
            %         Z     - Output of layer forward function
            
            numHeads = layer.NumHeads;
            X = varargin;

                    % Scaled dot-product attention
                    dropoutProb = 0; %% no dropout in predict
                    %%%% Q K V
                    A = scaledDotProductAttention(X{3},X{1},X{2},numHeads,dropoutProb);

                    % Concatenation
                    Z = iMergeHeads(A);
                   
        end

        function Z = forward(layer, varargin)
            % Forward input data through the layer at training
            % time and output the result and a memory value.
            %
            % Inputs:
            %         layer - Layer to forward propagate through 
            %         X     - Input data
            % Output:
            %         Z - Output of layer forward function
            
            numHeads = layer.NumHeads;
            X = varargin;
            % Scaled dot-product attention
                    dropoutProb = 0.2; %% no dropout in predict
                    %%%% Q K V
                    A = scaledDotProductAttention(X{3},X{1},X{2},numHeads,dropoutProb);

                    % Concatenation
                    Z = iMergeHeads(A);

        end

    end
end