function A = multiheadAttention(X,weights_K,bias_K,weights_V,bias_V,weights_Q,bias_Q,weights_A,bias_A,numHeads)
%MULTIHEADATTENTION Multi-head attention
% A = multiheadAttention(X,params) performs multi-head attention

% Linear
K = fullyconnect(X,weights_K,bias_K,DataFormat="CBT");
V = fullyconnect(X,weights_V,bias_V,DataFormat="CBT");
Q = fullyconnect(X,weights_Q,bias_Q,DataFormat="CBT");

% Scaled dot-product attention
A = scaledDotProductAttention(Q,K,V,numHeads);

% Concatenation
A = iMergeHeads(A);

% Linear
A = fullyconnect(A,weights_A,bias_A,DataFormat="CB");
A = reshape(A,[size(A,1),1,size(A,2)]);

    function A = scaledDotProductAttention(Q,K,V,numHeads)
        %SCALEDDOTPRODUCTATTENTION Scaled dot-product attention
        % A = scaledDotProductAttention(Q,K,V,numHeads)
        % performs scaled dot-product attention described in section 3.2.1 of [1].

        % The function is vectorized to apply attention to all heads
        % simultaneously.

        numFeatures = size(K,1);

        % Matrix multiplication
        K = iSplitHeadsQ(K,numFeatures,numHeads);
        Q = iSplitHeads(Q,numFeatures,numHeads);
        W = dlmtimes(K,Q);

        % Scale
        W = W./sqrt(size(Q,1));

        % Softmax
        W = softmax(W,DataFormat="CTUB");

        % Matrix multiplication
        V = iSplitHeads(V,numFeatures,numHeads);
        A = dlmtimes(V,W);

        function Z = iSplitHeads(X,numFeatures,numHeads)
            % For the key and value vectors, permute to put the dimension for
            % the heads last and the sequence length second. This enables
            % batched matrix multiplication to compute attention for all of the
            % heads at once.
            X = reshape(X,numFeatures/numHeads,numHeads,[],1);
            Z = permute(X,[1,3,2]);
        end
        function Z = iSplitHeadsQ(X,numFeatures,numHeads)
            % For the query vector, permute to put the dimension of the heads
            % last and the sequence length first. This enables batched matrix
            % multiplication to compute attention for all of the heads at once.
            X = reshape(X,numFeatures/numHeads,numHeads,[],1);
            Z = permute(X,[3,1,2]);
        end
    end
    function Z = iMergeHeads(X)
        % Merge the numFeatures-by-sequenceLength-by-numHeads array to a
        % (numFeatures*numHeads)-by-sequenceLength array
        X = permute(X,[1,3,2]);
        Z = reshape(X,size(X,1)*size(X,2),[]);
    end

end