%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Purpose: Run trained PCG classifier and obtain classifier outputs
% Inputs:
% 1. header
% 2. recordings
% 3. trained model
%
% Outputs:
% 1. score
% 2. label
% 3. classes
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [score, label, classes] = team_testing_code(header, recordings, loaded_model)
%% Extracting features

classes_murmur=loaded_model.classes_murmur;
classes_outcome=loaded_model.classes_outcome;

fs = 4000;
[transformed_features] = get_signal_features(header,recordings);
feature_out_overall = single.empty;
    for o = 1:4
        feature_out = getmswtfeat(transformed_features(o,:)',32,32,fs);
        feature_out_overall = [feature_out_overall;feature_out];
    end
extracted_features = feature_out_overall';

%% Model 1

%%%% Loop 1
model_murmur1_loop1 = loaded_model.model_murmur1{1,1};
model_murmur2_loop1 = loaded_model.model_murmur2{1,1};
[label_tmp_murmur_transformer1,score_murmur_transformer1] = classify(model_murmur1_loop1,extracted_features,'MiniBatchSize',32);
transformer_features_test = activations(model_murmur1_loop1,extracted_features,'gapl_block1');
transformer_features_test = transformer_features_test';
[label_tmp_murmur_NN1,score_murmur_NN1] = classify(model_murmur2_loop1,transformer_features_test,'MiniBatchSize',32);

%%%% Loop 2
model_murmur1_loop2 = loaded_model.model_murmur1{2,1};
model_murmur2_loop2 = loaded_model.model_murmur2{2,1};
[label_tmp_murmur_transformer2,score_murmur_transformer2] = classify(model_murmur1_loop2,extracted_features,'MiniBatchSize',32);
transformer_features_test = activations(model_murmur1_loop2,extracted_features,'gapl_block1');
transformer_features_test = transformer_features_test';
[label_tmp_murmur_NN2,score_murmur_NN2] = classify(model_murmur2_loop2,transformer_features_test,'MiniBatchSize',32);

%%%% Loop 3
model_murmur1_loop3 = loaded_model.model_murmur1{3,1};
model_murmur2_loop3 = loaded_model.model_murmur2{3,1};
[label_tmp_murmur_transformer3,score_murmur_transformer3] = classify(model_murmur1_loop3,extracted_features,'MiniBatchSize',32);
transformer_features_test = activations(model_murmur1_loop3,extracted_features,'gapl_block1');
transformer_features_test = transformer_features_test';
[label_tmp_murmur_NN3,score_murmur_NN3] = classify(model_murmur2_loop3,transformer_features_test,'MiniBatchSize',32);

%%%% Loop 4
model_murmur1_loop4 = loaded_model.model_murmur1{4,1};
model_murmur2_loop4 = loaded_model.model_murmur2{4,1};
[label_tmp_murmur_transformer4,score_murmur_transformer4] = classify(model_murmur1_loop4,extracted_features,'MiniBatchSize',32);
transformer_features_test = activations(model_murmur1_loop4,extracted_features,'gapl_block1');
transformer_features_test = transformer_features_test';
[label_tmp_murmur_NN4,score_murmur_NN4] = classify(model_murmur2_loop4,transformer_features_test,'MiniBatchSize',32);

%%%% Loop 5
model_murmur1_loop5 = loaded_model.model_murmur1{5,1};
model_murmur2_loop5 = loaded_model.model_murmur2{5,1};
[label_tmp_murmur_transformer5,score_murmur_transformer5] = classify(model_murmur1_loop5,extracted_features,'MiniBatchSize',32);
transformer_features_test = activations(model_murmur1_loop5,extracted_features,'gapl_block1');
transformer_features_test = transformer_features_test';
[label_tmp_murmur_NN5,score_murmur_NN5] = classify(model_murmur2_loop5,transformer_features_test,'MiniBatchSize',32);

%%%% Loop 6
model_murmur1_loop6 = loaded_model.model_murmur1{6,1};
model_murmur2_loop6 = loaded_model.model_murmur2{6,1};
[label_tmp_murmur_transformer6,score_murmur_transformer6] = classify(model_murmur1_loop6,extracted_features,'MiniBatchSize',32);
transformer_features_test = activations(model_murmur1_loop6,extracted_features,'gapl_block1');
transformer_features_test = transformer_features_test';
[label_tmp_murmur_NN6,score_murmur_NN6] = classify(model_murmur2_loop6,transformer_features_test,'MiniBatchSize',32);

%%%% Loop 7
model_murmur1_loop7 = loaded_model.model_murmur1{7,1};
model_murmur2_loop7 = loaded_model.model_murmur2{7,1};
[label_tmp_murmur_transformer7,score_murmur_transformer7] = classify(model_murmur1_loop7,extracted_features,'MiniBatchSize',32);
transformer_features_test = activations(model_murmur1_loop7,extracted_features,'gapl_block1');
transformer_features_test = transformer_features_test';
[label_tmp_murmur_NN7,score_murmur_NN7] = classify(model_murmur2_loop7,transformer_features_test,'MiniBatchSize',32);

%%%% Loop 8
model_murmur1_loop8 = loaded_model.model_murmur1{8,1};
model_murmur2_loop8 = loaded_model.model_murmur2{8,1};
[label_tmp_murmur_transformer8,score_murmur_transformer8] = classify(model_murmur1_loop8,extracted_features,'MiniBatchSize',32);
transformer_features_test = activations(model_murmur1_loop8,extracted_features,'gapl_block1');
transformer_features_test = transformer_features_test';
[label_tmp_murmur_NN8,score_murmur_NN8] = classify(model_murmur2_loop8,transformer_features_test,'MiniBatchSize',32);

%%%% Loop 9
model_murmur1_loop9 = loaded_model.model_murmur1{9,1};
model_murmur2_loop9 = loaded_model.model_murmur2{9,1};
[label_tmp_murmur_transformer9,score_murmur_transformer9] = classify(model_murmur1_loop9,extracted_features,'MiniBatchSize',32);
transformer_features_test = activations(model_murmur1_loop9,extracted_features,'gapl_block1');
transformer_features_test = transformer_features_test';
[label_tmp_murmur_NN9,score_murmur_NN9] = classify(model_murmur2_loop9,transformer_features_test,'MiniBatchSize',32);

%%%% Loop 10
model_murmur1_loop10 = loaded_model.model_murmur1{10,1};
model_murmur2_loop10 = loaded_model.model_murmur2{10,1};
[label_tmp_murmur_transformer10,score_murmur_transformer10] = classify(model_murmur1_loop10,extracted_features,'MiniBatchSize',32);
transformer_features_test = activations(model_murmur1_loop10,extracted_features,'gapl_block1');
transformer_features_test = transformer_features_test';
[label_tmp_murmur_NN10,score_murmur_NN10] = classify(model_murmur2_loop10,transformer_features_test,'MiniBatchSize',32);

%%%% Voting
final_scores_murmur = 1.*score_murmur_transformer1 + 1.*score_murmur_NN1 + 1.*score_murmur_transformer2 + 1.*score_murmur_NN2 + 1.*score_murmur_transformer3 + 1.*score_murmur_NN3 + 1.*score_murmur_transformer4 + 1.*score_murmur_NN4 + 1.*score_murmur_transformer5 + 1.*score_murmur_NN5 + 1.*score_murmur_transformer6 + 1.*score_murmur_NN6 + 1.*score_murmur_transformer7 + 1.*score_murmur_NN7 + 1.*score_murmur_transformer8 + 1.*score_murmur_NN8 + 1.*score_murmur_transformer9 + 1.*score_murmur_NN9 + 1.*score_murmur_transformer10 + 1.*score_murmur_NN10;
final_scores_murmur = final_scores_murmur./(sum(final_scores_murmur,2));
labels_final = categorical(["Absent";"Present";"Unknown"]);
voted_array = categorical;
[max_value,max_id] = max(final_scores_murmur);
voted_array = labels_final(max_id);
label_murmur=zeros(1,length(classes_murmur));
label_murmur(strmatch({char(voted_array)},classes_murmur))=1;

%% Model 2

%%%% Loop 1
model_outcome1_loop1 = loaded_model.model_outcome1{1,1};
model_outcome2_loop1 = loaded_model.model_outcome2{1,1};
[label_tmp_outcome_transformer1,score_outcome_transformer1] = classify(model_outcome1_loop1,extracted_features,'MiniBatchSize',32);
transformer_features_test = activations(model_outcome1_loop1,extracted_features,'gapl_block1');
transformer_features_test = transformer_features_test';
[label_tmp_outcome_NN1,score_outcome_NN1] = classify(model_outcome2_loop1,transformer_features_test,'MiniBatchSize',32);

%%%% Loop 2
model_outcome1_loop2 = loaded_model.model_outcome1{2,1};
model_outcome2_loop2 = loaded_model.model_outcome2{2,1};
[label_tmp_outcome_transformer2,score_outcome_transformer2] = classify(model_outcome1_loop2,extracted_features,'MiniBatchSize',32);
transformer_features_test = activations(model_outcome1_loop2,extracted_features,'gapl_block1');
transformer_features_test = transformer_features_test';
[label_tmp_outcome_NN2,score_outcome_NN2] = classify(model_outcome2_loop2,transformer_features_test,'MiniBatchSize',32);

%%%% Loop 3
model_outcome1_loop3 = loaded_model.model_outcome1{3,1};
model_outcome2_loop3 = loaded_model.model_outcome2{3,1};
[label_tmp_outcome_transformer3,score_outcome_transformer3] = classify(model_outcome1_loop3,extracted_features,'MiniBatchSize',32);
transformer_features_test = activations(model_outcome1_loop3,extracted_features,'gapl_block1');
transformer_features_test = transformer_features_test';
[label_tmp_outcome_NN3,score_outcome_NN3] = classify(model_outcome2_loop3,transformer_features_test,'MiniBatchSize',32);

%%%% Loop 4
model_outcome1_loop4 = loaded_model.model_outcome1{4,1};
model_outcome2_loop4 = loaded_model.model_outcome2{4,1};
[label_tmp_outcome_transformer4,score_outcome_transformer4] = classify(model_outcome1_loop4,extracted_features,'MiniBatchSize',32);
transformer_features_test = activations(model_outcome1_loop4,extracted_features,'gapl_block1');
transformer_features_test = transformer_features_test';
[label_tmp_outcome_NN4,score_outcome_NN4] = classify(model_outcome2_loop4,transformer_features_test,'MiniBatchSize',32);

%%%% Loop 5
model_outcome1_loop5 = loaded_model.model_outcome1{5,1};
model_outcome2_loop5 = loaded_model.model_outcome2{5,1};
[label_tmp_outcome_transformer5,score_outcome_transformer5] = classify(model_outcome1_loop5,extracted_features,'MiniBatchSize',32);
transformer_features_test = activations(model_outcome1_loop5,extracted_features,'gapl_block1');
transformer_features_test = transformer_features_test';
[label_tmp_outcome_NN5,score_outcome_NN5] = classify(model_outcome2_loop5,transformer_features_test,'MiniBatchSize',32);

%%%% Loop 6
model_outcome1_loop6 = loaded_model.model_outcome1{6,1};
model_outcome2_loop6 = loaded_model.model_outcome2{6,1};
[label_tmp_outcome_transformer6,score_outcome_transformer6] = classify(model_outcome1_loop6,extracted_features,'MiniBatchSize',32);
transformer_features_test = activations(model_outcome1_loop6,extracted_features,'gapl_block1');
transformer_features_test = transformer_features_test';
[label_tmp_outcome_NN6,score_outcome_NN6] = classify(model_outcome2_loop6,transformer_features_test,'MiniBatchSize',32);

%%%% Loop 7
model_outcome1_loop7 = loaded_model.model_outcome1{7,1};
model_outcome2_loop7 = loaded_model.model_outcome2{7,1};
[label_tmp_outcome_transformer7,score_outcome_transformer7] = classify(model_outcome1_loop7,extracted_features,'MiniBatchSize',32);
transformer_features_test = activations(model_outcome1_loop7,extracted_features,'gapl_block1');
transformer_features_test = transformer_features_test';
[label_tmp_outcome_NN7,score_outcome_NN7] = classify(model_outcome2_loop7,transformer_features_test,'MiniBatchSize',32);

%%%% Loop 8
model_outcome1_loop8 = loaded_model.model_outcome1{8,1};
model_outcome2_loop8 = loaded_model.model_outcome2{8,1};
[label_tmp_outcome_transformer8,score_outcome_transformer8] = classify(model_outcome1_loop8,extracted_features,'MiniBatchSize',32);
transformer_features_test = activations(model_outcome1_loop8,extracted_features,'gapl_block1');
transformer_features_test = transformer_features_test';
[label_tmp_outcome_NN8,score_outcome_NN8] = classify(model_outcome2_loop8,transformer_features_test,'MiniBatchSize',32);

%%%% Loop 9
model_outcome1_loop9 = loaded_model.model_outcome1{9,1};
model_outcome2_loop9 = loaded_model.model_outcome2{9,1};
[label_tmp_outcome_transformer9,score_outcome_transformer9] = classify(model_outcome1_loop9,extracted_features,'MiniBatchSize',32);
transformer_features_test = activations(model_outcome1_loop9,extracted_features,'gapl_block1');
transformer_features_test = transformer_features_test';
[label_tmp_outcome_NN9,score_outcome_NN9] = classify(model_outcome2_loop9,transformer_features_test,'MiniBatchSize',32);

%%%% Loop 10
model_outcome1_loop10 = loaded_model.model_outcome1{10,1};
model_outcome2_loop10 = loaded_model.model_outcome2{10,1};
[label_tmp_outcome_transformer10,score_outcome_transformer10] = classify(model_outcome1_loop10,extracted_features,'MiniBatchSize',32);
transformer_features_test = activations(model_outcome1_loop10,extracted_features,'gapl_block1');
transformer_features_test = transformer_features_test';
[label_tmp_outcome_NN10,score_outcome_NN10] = classify(model_outcome2_loop10,transformer_features_test,'MiniBatchSize',32);

%%%% Voting
final_scores_outcome = 1.*score_outcome_transformer1 + 1.*score_outcome_NN1 + 1.*score_outcome_transformer2 + 1.*score_outcome_NN2 + 1.*score_outcome_transformer3 + 1.*score_outcome_NN3 + 1.*score_outcome_transformer4 + 1.*score_outcome_NN4 + 1.*score_outcome_transformer5 + 1.*score_outcome_NN5 + 1.*score_outcome_transformer6 + 1.*score_outcome_NN6 + 1.*score_outcome_transformer7 + 1.*score_outcome_NN7 + 1.*score_outcome_transformer8 + 1.*score_outcome_NN8 + 1.*score_outcome_transformer9 + 1.*score_outcome_NN9 + 1.*score_outcome_transformer10 + 1.*score_outcome_NN10;
final_scores_outcome = final_scores_outcome./(sum(final_scores_outcome,2));
labels_final = categorical(["Abnormal";"Normal"]);
voted_array = categorical;
[max_value,max_id] = max(final_scores_outcome);
voted_array = labels_final(max_id);
label_outcome=zeros(1,length(classes_outcome));
label_outcome(strmatch({char(voted_array)},classes_outcome))=1;

%% Final labeling

classes=[classes_murmur classes_outcome];
score=[final_scores_murmur final_scores_outcome];

label=[label_murmur label_outcome];

end
