function model = team_training_code(input_directory,output_directory) % train_PCG_classifier
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Purpose: Train PCG classifiers and obtain the models
% Inputs:
% 1. input_directory
% 2. output_directory
%
% Outputs:
% 1. model: trained model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Find text files
patient_files=dir(fullfile(input_directory,'*.txt'));
patient_files={patient_files.name};
patient_files=sort(patient_files); % To help debugging
num_patient_files=length(patient_files);

fprintf('Loading data for %d patients...\n', num_patient_files)

% Extract classes from data
classes_murmur={};
classes_outcome={};
for j=1:num_patient_files

    current_class_murmur=get_class_murmur(fullfile(input_directory,patient_files{j}));
    classes_murmur=unique([classes_murmur current_class_murmur]);

    current_class_outcome=get_class_outcome(fullfile(input_directory,patient_files{j}));
    classes_outcome=unique([classes_outcome current_class_outcome]);

end

classes_murmur=sort(classes_murmur);
num_classes_murmur=length(classes_murmur);

classes_outcome=sort(classes_outcome);
num_classes_outcome=length(classes_outcome);

% Extracting features and labels
disp('Extracting features and labels...')

% demographic_features = single.empty;
transformed_features = cell.empty;
extracted_features = cell.empty;
labels_murmur = categorical;
labels_outcome = categorical;
% all_lengths = single.empty;
fs = 4000;
for j=1:num_patient_files

    fprintf('%d/%d \n',j,num_patient_files)

    current_header = get_header(fullfile(input_directory,patient_files{j}));
    current_recordings = load_recordings(input_directory,current_header);
%     demographic_features(j,:) = get_demographics(current_header);
    [transformed_features{j,1}] = get_signal_features(current_header,current_recordings);

    feature_out_overall = single.empty;
    for o = 1:4
        feature_out = getmswtfeat(transformed_features{j,1}(o,:)',32,32,fs);
        feature_out_overall = [feature_out_overall;feature_out];
    end
    extracted_features{j,1} = feature_out_overall';

    labels_murmur(j,1) = get_class_murmur(fullfile(input_directory,patient_files{j}));
    labels_outcome(j,1) = get_class_outcome(fullfile(input_directory,patient_files{j}));
    
end
labels_murmur = reordercats(labels_murmur,{'Absent','Present','Unknown'});
labels_outcome = reordercats(labels_outcome,{'Abnormal','Normal'});

%% train network

%%%%%%%%%%%%%%%%%%%%%%%%%% MODEL 1 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp('Training model 1...')

train_signals = extracted_features;
Y_train = labels_murmur;

model_murmur1 = cell.empty;
model_murmur2 = cell.empty;
for loops_id = 1:10
%%%%%%%%% Weights
Y_train2 = double(Y_train);
total_length = length(Y_train);
weights = double.empty;
for k = 1:length(unique(Y_train2))
length_selected = length(find(Y_train2 == k));
weights(k,1) = 1 - (length_selected/total_length);
end

%%%%%%%%% NETWORK
NumFeatures = size(train_signals{1,1},1);
NumSignals = size(train_signals{1,1},1);
[lgraph] = transformer_network_bigger(NumSignals,Y_train,weights);

miniBatchSize = 32;
options = trainingOptions("adam", ...
    ExecutionEnvironment="gpu", ...
    MiniBatchSize=miniBatchSize, ...
    MaxEpochs=90, ...
    SequencePaddingDirection="right", ...
    InitialLearnRate=0.001, ...
    LearnRateSchedule="piecewise", ...
    LearnRateDropFactor=0.1, ...
    LearnRateDropPeriod=65, ...
    L2Regularization=0.0001, ...
    Shuffle='every-epoch', ...
    Verbose=0);

rng('default')
clear net
[net,info] = trainNetwork(train_signals,Y_train,lgraph,options);

model_murmur1{loops_id,1} = net;

transformer_features = activations(model_murmur1{loops_id,1},train_signals,'gapl_block1');
num2Add = [1500,0,1500];
[transformer_features_edited,Y_train_edited] = SMOTE_transformer(transformer_features,Y_train,num2Add);

numFeatures2 = size(transformer_features_edited,2);
layers2 = [
    featureInputLayer(numFeatures2,'Name','input',Normalization="zscore")
    fullyConnectedLayer(3, 'Name','fc')
    softmaxLayer('Name','sm')
    classificationLayer
    ];

options = trainingOptions('adam', ...
    'ExecutionEnvironment','gpu', ...
    'MaxEpochs',200, ...
    'MiniBatchSize',32, ...
    'InitialLearnRate',0.001, ...
    'LearnRateSchedule',"piecewise", ...
    'LearnRateDropFactor',0.1, ...
    'LearnRateDropPeriod',180, ...
    'L2Regularization',0.0001, ...
    'Shuffle','every-epoch', ...
    'Verbose',false);

rng('default')
clear net
[net info] = trainNetwork(transformer_features_edited,Y_train_edited,layers2,options);

model_murmur2{loops_id,1} = net;

end

%%%%%%%%%%%%%%%%%%%%%%%%%% MODEL 2 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp('Training model 2...')

train_signals = extracted_features;
Y_train = labels_outcome;

model_outcome1 = cell.empty;
model_outcome2 = cell.empty;
for loops_id = 1:10
%%%%%%%%% Weights
Y_train2 = double(Y_train);
total_length = length(Y_train);
weights = double.empty;
for k = 1:length(unique(Y_train2))
length_selected = length(find(Y_train2 == k));
weights(k,1) = 1 - (length_selected/total_length);
end

%%%%%%%%% NETWORK
NumFeatures = size(train_signals{1,1},1);
NumSignals = size(train_signals{1,1},1);
[lgraph] = transformer_network_bigger(NumSignals,Y_train,weights);

miniBatchSize = 32;
options = trainingOptions("adam", ...
    ExecutionEnvironment="gpu", ...
    MiniBatchSize=miniBatchSize, ...
    MaxEpochs=60, ...
    SequencePaddingDirection="right", ...
    InitialLearnRate=0.001, ...
    LearnRateSchedule="piecewise", ...
    LearnRateDropFactor=0.1, ...
    LearnRateDropPeriod=40, ...
    L2Regularization=0.0001, ...
    Shuffle='every-epoch', ...
    Verbose=0);

rng('default')
clear net
[net,info] = trainNetwork(train_signals,Y_train,lgraph,options);

model_outcome1{loops_id,1} = net;

transformer_features = activations(model_murmur1{loops_id,1},train_signals,'gapl_block1');
num2Add = [3500,3500];
[transformer_features_edited,Y_train_edited] = SMOTE_transformer(transformer_features,Y_train,num2Add);

numFeatures2 = size(transformer_features_edited,2);
layers2 = [
    featureInputLayer(numFeatures2,'Name','input',Normalization="zscore")
    fullyConnectedLayer(2, 'Name','fc')
    softmaxLayer('Name','sm')
    classificationLayer
    ];

options = trainingOptions('adam', ...
    'ExecutionEnvironment','gpu', ...
    'MaxEpochs',120, ... %120
    'MiniBatchSize',32, ...
    'InitialLearnRate',0.001, ...
    'LearnRateSchedule',"piecewise", ...
    'LearnRateDropFactor',0.1, ...
    'LearnRateDropPeriod',100, ... %100
    'L2Regularization',0.0001, ...
    'Shuffle','every-epoch', ...
    'Verbose',false);

rng('default')
clear net
[net info] = trainNetwork(transformer_features_edited,Y_train_edited,layers2,options);

model_outcome2{loops_id,1} = net;

end

filename = fullfile(output_directory,'model.mat');
save(filename,'model_murmur1','model_murmur2','classes_murmur','model_outcome1','model_outcome2','classes_outcome','-v7.3');

end

function save_model(model_murmur,classes_murmur,model_outcome,classes_outcome,output_directory) %save_PCG_model
% Save results.
filename = fullfile(output_directory,'model.mat');
save(filename,'model_murmur','classes_murmur','model_outcome','classes_outcome','-v7.3');

disp('Done.')
end

function class=get_class_murmur(input_header)

current_header=get_header(input_header);

class=current_header(startsWith(current_header,'#Murmur'));
class=strsplit(class{1},':');
class=strtrim(class{2});

end

function class=get_class_outcome(input_header)

current_header=get_header(input_header);

class=current_header(startsWith(current_header,'#Outcome'));
class=strsplit(class{1},':');
class=strtrim(class{2});

end

function current_header=get_header(input_header)

current_header=fileread(input_header);
current_header=strsplit(current_header,'\n');

end

function current_recordings=load_recordings(input_directory,current_header)

recording_files=get_recording_files(current_header);

current_recordings={};

for j=1:length(recording_files)

    current_recordings{j}=audioread(fullfile(input_directory,strtrim(recording_files{j})));

end

end

function recording_files=get_recording_files(current_header)

recording_files={};

num_locations=strsplit(current_header{1},' ');
num_locations=str2double(num_locations{2});

for j=2:num_locations+1

    current_line=strsplit(current_header{j},' ');
    recording_files{j-1}=current_line{3};

end

end
