function model = team_training_code_work(input_directory,output_directory) % train_PCG_classifier
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Purpose: Train PCG classifiers and obtain the models
% Inputs:
% 1. input_directory
% 2. output_directory
%
% Outputs:
% 1. model: trained model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Find text files
patient_files=dir(fullfile(input_directory,'*.txt'));
patient_files={patient_files.name};
patient_files=sort(patient_files); % To help debugging
num_patient_files=length(patient_files);

fprintf('Loading data for %d patients...\n', num_patient_files)

% Extract classes from data
classes_murmur={};
classes_outcome={};
for j=1:num_patient_files

    current_class_murmur=get_class_murmur(fullfile(input_directory,patient_files{j}));
    classes_murmur=unique([classes_murmur current_class_murmur]);

    current_class_outcome=get_class_outcome(fullfile(input_directory,patient_files{j}));
    classes_outcome=unique([classes_outcome current_class_outcome]);

end

classes_murmur=sort(classes_murmur);
num_classes_murmur=length(classes_murmur);

classes_outcome=sort(classes_outcome);
num_classes_outcome=length(classes_outcome);

% Extracting features and labels
disp('Extracting features and labels...')

demographic_features = single.empty;
transformed_features = cell.empty;
extracted_features = cell.empty;
labels_murmur = categorical;
labels_outcome = categorical;
all_lengths = single.empty;
fs = 4000;
for j=1:num_patient_files

    fprintf('%d/%d \n',j,num_patient_files)

    current_header = get_header(fullfile(input_directory,patient_files{j}));
    current_recordings = load_recordings(input_directory,current_header);
    [transformed_features{j,1}] = get_signal_features(current_header,current_recordings);
    
    feature_out_overall = single.empty;
    for o = 1:4
        feature_out = getmswtfeat(transformed_features{j,1}(o,:)',32,32,fs);
        feature_out_overall = [feature_out_overall;feature_out];
    end
    extracted_features{j,1} = feature_out_overall';

    labels_murmur(j,1) = get_class_murmur(fullfile(input_directory,patient_files{j}));
    labels_outcome(j,1) = get_class_outcome(fullfile(input_directory,patient_files{j}));
    
end
labels_murmur = reordercats(labels_murmur,{'Absent','Present','Unknown'});
labels_outcome = reordercats(labels_outcome,{'Abnormal','Normal'});

% % % %%%%%%%%%%% Normalization
% % % demographic_features2 = normalize(demographic_features,1);
% % % 
% % % transformed_features2 = transformed_features;
% % % for j = 1:num_patient_files
% % % extra_demog_vector = [demographic_features2(j,:),zeros(1,40000-length(demographic_features2(j,:)))];
% % % transformed_features2{j,1} = [transformed_features2{j,1};extra_demog_vector];
% % % end

% 
% transformed_features2 = transformed_features;
% for j = 1:num_patient_files
% extra_vector = [extracted_features{j,1},zeros(30,40000-length(extracted_features{j,1}))];
% transformed_features2{j,1} = [transformed_features2{j,1};extra_vector];
% end

extracted_features_mean = single.empty;
for k = 1:length(extracted_features)
    X = extracted_features{k,1};
    extracted_features_mean(k,:) = mean(X,2);
end
    [idx,scores_transformer] = fscchi2(extracted_features_mean,labels_murmur);
    figure
    bar(scores_transformer(idx))
    xlabel('Predictor rank')
    ylabel('Predictor importance score')

extracted_features2 = cell.empty;
for k = 1:length(extracted_features)
    X = extracted_features{k,1};
    X2 = X(idx(1:12),:);
    extracted_features2{k,1} = X2;
end

% handcrafted_features = single.empty;
% for j=1:num_patient_files
% 
%     fprintf('%d/%d \n',j,num_patient_files)
%     
%         selected_subject = transformed_features{j,1};
%         heartRate = single.empty;
%         systolicTimeInterval = single.empty;
%         systolediastole_features = single.empty;
%         for o = 1:4
%             [heartRate(o,1),systolicTimeInterval(o,1)] = getHeartRateSchmidt(double(selected_subject(o,:)), 4000, 0);
%             [d_distributions,max_S1,min_S1,max_S2,min_S2,max_systole,min_systole,max_diastole,min_diastole] = get_duration_distributions(heartRate(o,1),systolicTimeInterval(o,1));
%             systolediastole_features = [systolediastole_features,max_S1,min_S1,max_S2,min_S2,max_systole,min_systole,max_diastole,min_diastole];
%         end
%         features = single.empty;
%         for o = 1:4
% %             if (j == 156) && (o == 1)
% %                continue
% %             end
% %             if (j == 331) && (o == 2)
% %                continue
% %             end
% %             if (j == 722) && (o == 4)
% %                continue
% %             end
%             assigned_states = runSpringerSegmentationAlgorithm(double(selected_subject(o,5:end))',... 
%                                                                     4000,... 
%                                                                     Springer_B_matrix, Springer_pi_vector,...
%                                                                     Springer_total_obs_distribution, false);
%             % get states
%             idx_states = get_states(assigned_states);
%             features_time = get_features_time(double(selected_subject(o,:)),idx_states);
%             features_freq = get_features_frequency(double(selected_subject(o,:)),idx_states);
%             features = [features, features_time, features_freq];
%         end
% 
% % if (j == 156) || (j == 331) || (j == 722)
% %    features = [features, features_time, features_freq];
% % end
% 
% handcrafted_features(j,:) = [heartRate',systolicTimeInterval',systolediastole_features,features];
% 
% end

%% train RF

disp('Training the model...')

folds = cvpartition(length(labels_murmur),'kfold',10);

all_nets = cell.empty;
test_idx = double.empty;
all_scores = cell.empty;
all_accuracies = single.empty;
all_predicitons = cell.empty;
all_original = cell.empty;
all_scores2 = single.empty;
all_accuracies2 = single.empty;
all_predicitons2 = categorical;
all_original2 = categorical;
array = string;
array2 = string;
all_predicitons_transformer = cell.empty;
all_predicitons_NN = cell.empty;
all_predicitons_RF = cell.empty;
all_predicitons_SVM = cell.empty;
all_predicitons_ADA = cell.empty;
all_predicitons_kNN = cell.empty;
all_predicitons_voting = cell.empty;
all_predicitons_trained_voting = cell.empty;
all_scores_transformer = cell.empty;
all_scores_NN = cell.empty;
all_scores_RF = cell.empty;
all_scores_SVM = cell.empty;
all_scores_ADA = cell.empty;
all_scores_kNN = cell.empty;
all_scores_voting = cell.empty;
all_scores_trained_voting = cell.empty;
for i = 1:folds.NumTestSets
i
ttest = test(folds,i);
ttrain = training(folds,i);
% test_idx(i,1) = find(test(folds,i) == 1);

train_signals = extracted_features(ttrain,1);
test_signals = extracted_features(ttest,1);
% 
% train_demog = demographic_features(ttrain,:);
% test_demog = demographic_features(ttest,:);

Y_train = labels_murmur(ttrain);
Y_test = labels_murmur(ttest);

%%%%%%%%% Weights
Y_train2 = double(Y_train);
total_length = length(Y_train);
weights = double.empty;
for k = 1:length(unique(Y_train2))
length_selected = length(find(Y_train2 == k));
weights(k,1) = 1 - (length_selected/total_length);
end

%%%%%%%%% NETWORKS
% TRANSFORMER
[net_transformer,predictedLabels_transformer,scores_transformer] = train_transformer(train_signals,Y_train,test_signals,Y_test,weights);
test_acc_transformer = (sum(categorical(predictedLabels_transformer) == Y_test)./numel(Y_test)).*100;
% figure;confusionchart(Y_test,predictedLabels_transformer);
all_predicitons_transformer{i,1} = predictedLabels_transformer;
all_scores_transformer{i,1} = scores_transformer;

%%%%%%%%%%%%%%%%% SMOTE
transformer_features_train = activations(net_transformer,train_signals,'gapl_block1');
transformer_features_test = activations(net_transformer,test_signals,'gapl_block1');
transformer_features_test = transformer_features_test';


num2Add = [1500,0,1500];
% num2Add = [3500,3500];
[transformer_features_train_edited,Y_train_edited] = SMOTE_transformer(transformer_features_train,Y_train,num2Add);

% Neural network
[net_NN,predictedLabels_NN,scores_NN] = train_NN(transformer_features_train_edited,Y_train_edited,transformer_features_test,Y_test);
test_acc_NN = (sum(categorical(predictedLabels_NN) == Y_test)./numel(Y_test)).*100;
% figure;confusionchart(Y_test,predictedLabels_NN);
all_predicitons_NN{i,1} = predictedLabels_NN;
all_scores_NN{i,1} = scores_NN;



% %%%%%%%%% NETWORKS
% % TRANSFORMER
% [net_transformer2,predictedLabels_transformer2,scores_transformer2] = train_transformer(train_signals,Y_train,test_signals,Y_test,weights);
% test_acc_transformer2 = (sum(categorical(predictedLabels_transformer2) == Y_test)./numel(Y_test)).*100;
% % figure;confusionchart(Y_test,predictedLabels_transformer2);
% all_predicitons_transformer{i,1} = predictedLabels_transformer2;
% all_scores_transformer{i,1} = scores_transformer2;
% 
% %%%%%%%%%%%%%%%%% SMOTE
% transformer_features_train = activations(net_transformer2,train_signals,'gapl_block1');
% transformer_features_test = activations(net_transformer2,test_signals,'gapl_block1');
% transformer_features_test = transformer_features_test';
% 
% num2Add = [1500,0,1500];
% % num2Add = [3500,3500];
% [transformer_features_train_edited,Y_train_edited] = SMOTE_transformer(transformer_features_train,Y_train,num2Add);
% 
% % Neural network
% [net_NN2,predictedLabels_NN2,scores_NN2] = train_NN(transformer_features_train_edited,Y_train_edited,transformer_features_test,Y_test);
% test_acc_NN2 = (sum(categorical(predictedLabels_NN2) == Y_test)./numel(Y_test)).*100;
% figure;confusionchart(Y_test,predictedLabels_NN2);
% all_predicitons_NN{i,1} = predictedLabels_NN2;
% all_scores_NN{i,1} = scores_NN2;
% 
% %%%%%%%%% NETWORKS
% % TRANSFORMER
% [net_transformer3,predictedLabels_transformer3,scores_transformer3] = train_transformer(train_signals,Y_train,test_signals,Y_test,weights);
% test_acc_transformer3 = (sum(categorical(predictedLabels_transformer3) == Y_test)./numel(Y_test)).*100;
% % figure;confusionchart(Y_test,predictedLabels_transformer3);
% all_predicitons_transformer{i,1} = predictedLabels_transformer3;
% all_scores_transformer{i,1} = scores_transformer3;
% 
% %%%%%%%%%%%%%%%%% SMOTE
% transformer_features_train = activations(net_transformer3,train_signals,'gapl_block1');
% transformer_features_test = activations(net_transformer3,test_signals,'gapl_block1');
% transformer_features_test = transformer_features_test';
% 
% num2Add = [1500,0,1500];
% % num2Add = [3500,3500];
% [transformer_features_train_edited,Y_train_edited] = SMOTE_transformer(transformer_features_train,Y_train,num2Add);
% 
% % Neural network
% [net_NN3,predictedLabels_NN3,scores_NN3] = train_NN(transformer_features_train_edited,Y_train_edited,transformer_features_test,Y_test);
% test_acc_NN3 = (sum(categorical(predictedLabels_NN3) == Y_test)./numel(Y_test)).*100;
% % figure;confusionchart(Y_test,predictedLabels_NN3);
% all_predicitons_NN{i,1} = predictedLabels_NN3;
% all_scores_NN{i,1} = scores_NN3;
% 
% %%%%%%%%% NETWORKS
% % TRANSFORMER
% [net_transformer4,predictedLabels_transformer4,scores_transformer4] = train_transformer(train_signals,Y_train,test_signals,Y_test,weights);
% test_acc_transformer4 = (sum(categorical(predictedLabels_transformer4) == Y_test)./numel(Y_test)).*100;
% % figure;confusionchart(Y_test,predictedLabels_transformer4);
% all_predicitons_transformer{i,1} = predictedLabels_transformer4;
% all_scores_transformer{i,1} = scores_transformer4;
% 
% %%%%%%%%%%%%%%%%% SMOTE
% transformer_features_train = activations(net_transformer4,train_signals,'gapl_block1');
% transformer_features_test = activations(net_transformer4,test_signals,'gapl_block1');
% transformer_features_test = transformer_features_test';
% 
% num2Add = [1500,0,1500];
% % num2Add = [3500,3500];
% [transformer_features_train_edited,Y_train_edited] = SMOTE_transformer(transformer_features_train,Y_train,num2Add);
% 
% % Neural network
% [net_NN4,predictedLabels_NN4,scores_NN4] = train_NN(transformer_features_train_edited,Y_train_edited,transformer_features_test,Y_test);
% test_acc_NN4 = (sum(categorical(predictedLabels_NN4) == Y_test)./numel(Y_test)).*100;
% % figure;confusionchart(Y_test,predictedLabels_NN4);
% all_predicitons_NN{i,1} = predictedLabels_NN4;
% all_scores_NN{i,1} = scores_NN4;
% 
% 
% %%%%%%%%% NETWORKS
% % TRANSFORMER
% [net_transformer5,predictedLabels_transformer5,scores_transformer5] = train_transformer(train_signals,Y_train,test_signals,Y_test,weights);
% test_acc_transformer5 = (sum(categorical(predictedLabels_transformer5) == Y_test)./numel(Y_test)).*100;
% % figure;confusionchart(Y_test,predictedLabels_transformer5);
% all_predicitons_transformer{i,1} = predictedLabels_transformer5;
% all_scores_transformer{i,1} = scores_transformer5;
% 
% %%%%%%%%%%%%%%%%% SMOTE
% transformer_features_train = activations(net_transformer5,train_signals,'gapl_block1');
% transformer_features_test = activations(net_transformer5,test_signals,'gapl_block1');
% transformer_features_test = transformer_features_test';
% 
% num2Add = [1500,0,1500];
% % num2Add = [3500,3500];
% [transformer_features_train_edited,Y_train_edited] = SMOTE_transformer(transformer_features_train,Y_train,num2Add);
% 
% % Neural network
% [net_NN5,predictedLabels_NN5,scores_NN5] = train_NN(transformer_features_train_edited,Y_train_edited,transformer_features_test,Y_test);
% test_acc_NN5 = (sum(categorical(predictedLabels_NN5) == Y_test)./numel(Y_test)).*100;
% % figure;confusionchart(Y_test,predictedLabels_NN5);
% all_predicitons_NN{i,1} = predictedLabels_NN5;
% all_scores_NN{i,1} = scores_NN5;
% 
% 
% VOTING
final_scores = 1.*scores_transformer + 1.*scores_NN;% + 1.*scores_transformer2 + 1.*scores_NN2 + 1.*scores_transformer3 + 1.*scores_NN3;
final_scores = final_scores./(sum(final_scores,2));
labels_final = categorical(["Absent";"Present";"Unknown"]);
% labels_final = categorical(["Abnormal";"Normal"]);
voted_array = categorical;
for h = 1:size(final_scores,1)
    [max_value,max_id] = max(final_scores(h,:));
    voted_array(h,1) = labels_final(max_id);
end
test_acc_final = (sum(categorical(voted_array) == Y_test)./numel(Y_test)).*100;
% figure;confusionchart(Y_test,voted_array);
all_predicitons_voting{i,1} = voted_array;
all_scores_voting{i,1} = final_scores;

% % VOTING
% final_scores2 = 1.*scores_transformer + 1.*scores_NN + 1.*scores_transformer2 + 1.*scores_NN2;% + 1.*scores_transformer3 + 1.*scores_NN3;
% final_scores2 = final_scores2./(sum(final_scores2,2));
% labels_final = categorical(["Absent";"Present";"Unknown"]);
% % labels_final = categorical(["Abnormal";"Normal"]);
% voted_array2 = categorical;
% for h = 1:size(final_scores2,1)
%     [max_value,max_id] = max(final_scores2(h,:));
%     voted_array2(h,1) = labels_final(max_id);
% end
% test_acc_final2 = (sum(categorical(voted_array2) == Y_test)./numel(Y_test)).*100;
% % figure;confusionchart(Y_test,voted_array);
% all_predicitons_voting2{i,1} = voted_array2;
% all_scores_voting2{i,1} = final_scores2;

% % VOTING
% final_scores3 = 1.*scores_transformer + 1.*scores_NN + 1.*scores_transformer2 + 1.*scores_NN2 + 1.*scores_transformer3 + 1.*scores_NN3;
% final_scores3 = final_scores3./(sum(final_scores3,2));
% labels_final = categorical(["Absent";"Present";"Unknown"]);
% % labels_final = categorical(["Abnormal";"Normal"]);
% voted_array3 = categorical;
% for h = 1:size(final_scores3,1)
%     [max_value,max_id] = max(final_scores3(h,:));
%     voted_array3(h,1) = labels_final(max_id);
% end
% test_acc_final3 = (sum(categorical(voted_array3) == Y_test)./numel(Y_test)).*100;
% % figure;confusionchart(Y_test,voted_array);
% all_predicitons_voting3{i,1} = voted_array3;
% all_scores_voting3{i,1} = final_scores3;
% 
% % VOTING
% final_scores4 = 1.*scores_transformer + 1.*scores_NN + 1.*scores_transformer2 + 1.*scores_NN2 + 1.*scores_transformer3 + 1.*scores_NN3 + 1.*scores_transformer4 + 1.*scores_NN4;
% final_scores4 = final_scores4./(sum(final_scores4,2));
% labels_final = categorical(["Absent";"Present";"Unknown"]);
% % labels_final = categorical(["Abnormal";"Normal"]);
% voted_array4 = categorical;
% for h = 1:size(final_scores4,1)
%     [max_value,max_id] = max(final_scores4(h,:));
%     voted_array4(h,1) = labels_final(max_id);
% end
% test_acc_final4 = (sum(categorical(voted_array4) == Y_test)./numel(Y_test)).*100;
% % figure;confusionchart(Y_test,voted_array);
% all_predicitons_voting4{i,1} = voted_array4;
% all_scores_voting4{i,1} = final_scores4;
% 
% % VOTING
% final_scores5 = 1.*scores_transformer + 1.*scores_NN + 1.*scores_transformer2 + 1.*scores_NN2 + 1.*scores_transformer3 + 1.*scores_NN3 + 1.*scores_transformer4 + 1.*scores_NN4 + 1.*scores_transformer5 + 1.*scores_NN5;
% final_scores5 = final_scores5./(sum(final_scores5,2));
% labels_final = categorical(["Absent";"Present";"Unknown"]);
% % labels_final = categorical(["Abnormal";"Normal"]);
% voted_array5 = categorical;
% for h = 1:size(final_scores5,1)
%     [max_value,max_id] = max(final_scores5(h,:));
%     voted_array5(h,1) = labels_final(max_id);
% end
% test_acc_final5 = (sum(categorical(voted_array5) == Y_test)./numel(Y_test)).*100;
% % figure;confusionchart(Y_test,voted_array);
% all_predicitons_voting5{i,1} = voted_array5;
% all_scores_voting5{i,1} = final_scores5;

%%
figure("Position",[524,73,745,889]);

subplot(3,2,1)
cm = confusionchart(Y_test,predictedLabels_transformer);
title(['Transformer Accuracy: ',num2str(round(test_acc_transformer,2))]);
subplot(3,2,2)
hold on
box on
grid on
[X,Y,T,AUC1] = perfcurve(Y_test,scores_transformer(:,1),'Absent');
plot(X,Y,'g')
[X,Y,T,AUC2] = perfcurve(Y_test,scores_transformer(:,2),'Present');
plot(X,Y,'r')
[X,Y,T,AUC3] = perfcurve(Y_test,scores_transformer(:,3),'Unknown');
plot(X,Y,'k')
legend(['Absent AUC: ',num2str(round(AUC1,3))],['Present AUC: ',num2str(round(AUC2,3))],['Unknown AUC: ',num2str(round(AUC3,3))],...
        'Location','southeast');
xlabel('False positive rate');
ylabel('True positive rate');

subplot(3,2,3)
cm = confusionchart(Y_test,predictedLabels_NN);
title(['NN Accuracy: ',num2str(round(test_acc_NN,2))]);
subplot(3,2,4)
hold on
box on
grid on
[X,Y,T,AUC1] = perfcurve(Y_test,scores_NN(:,1),'Absent');
plot(X,Y,'g')
[X,Y,T,AUC2] = perfcurve(Y_test,scores_NN(:,2),'Present');
plot(X,Y,'r')
[X,Y,T,AUC3] = perfcurve(Y_test,scores_NN(:,3),'Unknown');
plot(X,Y,'k')
legend(['Absent AUC: ',num2str(round(AUC1,3))],['Present AUC: ',num2str(round(AUC2,3))],['Unknown AUC: ',num2str(round(AUC3,3))],...
        'Location','southeast');
xlabel('False positive rate');
ylabel('True positive rate');

subplot(3,2,5)
cm = confusionchart(Y_test,voted_array);
title(['Average Voting Accuracy: ',num2str(round(test_acc_final,2))]);
subplot(3,2,6)
hold on
box on
grid on
[X,Y,T,AUC1] = perfcurve(Y_test,final_scores(:,1),'Absent');
plot(X,Y,'g')
[X,Y,T,AUC2] = perfcurve(Y_test,final_scores(:,2),'Present');
plot(X,Y,'r')
[X,Y,T,AUC3] = perfcurve(Y_test,final_scores(:,3),'Unknown');
plot(X,Y,'k')
legend(['Absent AUC: ',num2str(round(AUC1,3))],['Present AUC: ',num2str(round(AUC2,3))],['Unknown AUC: ',num2str(round(AUC3,3))],...
        'Location','southeast');
xlabel('False positive rate');
ylabel('True positive rate');

% % % % % % % figure("Position",[524,73,745,889]);
% % % % % % % 
% % % % % % % subplot(5,2,1)
% % % % % % % cm = confusionchart(Y_test,voted_array);
% % % % % % % title(['Average Voting 1 Accuracy: ',num2str(round(test_acc_final,2))]);
% % % % % % % subplot(5,2,2)
% % % % % % % hold on
% % % % % % % box on
% % % % % % % grid on
% % % % % % % [X,Y,T,AUC1] = perfcurve(Y_test,final_scores(:,1),'Absent');
% % % % % % % plot(X,Y,'g')
% % % % % % % [X,Y,T,AUC2] = perfcurve(Y_test,final_scores(:,2),'Present');
% % % % % % % plot(X,Y,'r')
% % % % % % % [X,Y,T,AUC3] = perfcurve(Y_test,final_scores(:,3),'Unknown');
% % % % % % % plot(X,Y,'k')
% % % % % % % legend(['Absent AUC: ',num2str(round(AUC1,3))],['Present AUC: ',num2str(round(AUC2,3))],['Unknown AUC: ',num2str(round(AUC3,3))],...
% % % % % % %         'Location','southeast');
% % % % % % % xlabel('False positive rate');
% % % % % % % ylabel('True positive rate');
% % % % % % % 
% % % % % % % subplot(5,2,3)
% % % % % % % cm = confusionchart(Y_test,voted_array2);
% % % % % % % title(['Average Voting 2 Accuracy: ',num2str(round(test_acc_final2,2))]);
% % % % % % % subplot(5,2,4)
% % % % % % % hold on
% % % % % % % box on
% % % % % % % grid on
% % % % % % % [X,Y,T,AUC1] = perfcurve(Y_test,final_scores2(:,1),'Absent');
% % % % % % % plot(X,Y,'g')
% % % % % % % [X,Y,T,AUC2] = perfcurve(Y_test,final_scores2(:,2),'Present');
% % % % % % % plot(X,Y,'r')
% % % % % % % [X,Y,T,AUC3] = perfcurve(Y_test,final_scores2(:,3),'Unknown');
% % % % % % % plot(X,Y,'k')
% % % % % % % legend(['Absent AUC: ',num2str(round(AUC1,3))],['Present AUC: ',num2str(round(AUC2,3))],['Unknown AUC: ',num2str(round(AUC3,3))],...
% % % % % % %         'Location','southeast');
% % % % % % % xlabel('False positive rate');
% % % % % % % ylabel('True positive rate');
% % % % % % % 
% % % % % % % subplot(5,2,5)
% % % % % % % cm = confusionchart(Y_test,voted_array3);
% % % % % % % title(['Average Voting 3 Accuracy: ',num2str(round(test_acc_final3,2))]);
% % % % % % % subplot(5,2,6)
% % % % % % % hold on
% % % % % % % box on
% % % % % % % grid on
% % % % % % % [X,Y,T,AUC1] = perfcurve(Y_test,final_scores3(:,1),'Absent');
% % % % % % % plot(X,Y,'g')
% % % % % % % [X,Y,T,AUC2] = perfcurve(Y_test,final_scores3(:,2),'Present');
% % % % % % % plot(X,Y,'r')
% % % % % % % [X,Y,T,AUC3] = perfcurve(Y_test,final_scores3(:,3),'Unknown');
% % % % % % % plot(X,Y,'k')
% % % % % % % legend(['Absent AUC: ',num2str(round(AUC1,3))],['Present AUC: ',num2str(round(AUC2,3))],['Unknown AUC: ',num2str(round(AUC3,3))],...
% % % % % % %         'Location','southeast');
% % % % % % % xlabel('False positive rate');
% % % % % % % ylabel('True positive rate');
% % % % % % % 
% % % % % % % subplot(5,2,7)
% % % % % % % cm = confusionchart(Y_test,voted_array4);
% % % % % % % title(['Average Voting 4 Accuracy: ',num2str(round(test_acc_final4,2))]);
% % % % % % % subplot(5,2,8)
% % % % % % % hold on
% % % % % % % box on
% % % % % % % grid on
% % % % % % % [X,Y,T,AUC1] = perfcurve(Y_test,final_scores4(:,1),'Absent');
% % % % % % % plot(X,Y,'g')
% % % % % % % [X,Y,T,AUC2] = perfcurve(Y_test,final_scores4(:,2),'Present');
% % % % % % % plot(X,Y,'r')
% % % % % % % [X,Y,T,AUC3] = perfcurve(Y_test,final_scores4(:,3),'Unknown');
% % % % % % % plot(X,Y,'k')
% % % % % % % legend(['Absent AUC: ',num2str(round(AUC1,3))],['Present AUC: ',num2str(round(AUC2,3))],['Unknown AUC: ',num2str(round(AUC3,3))],...
% % % % % % %         'Location','southeast');
% % % % % % % xlabel('False positive rate');
% % % % % % % ylabel('True positive rate');
% % % % % % % 
% % % % % % % subplot(5,2,9)
% % % % % % % cm = confusionchart(Y_test,voted_array5);
% % % % % % % title(['Average Voting 5 Accuracy: ',num2str(round(test_acc_final5,2))]);
% % % % % % % subplot(5,2,10)
% % % % % % % hold on
% % % % % % % box on
% % % % % % % grid on
% % % % % % % [X,Y,T,AUC1] = perfcurve(Y_test,final_scores5(:,1),'Absent');
% % % % % % % plot(X,Y,'g')
% % % % % % % [X,Y,T,AUC2] = perfcurve(Y_test,final_scores5(:,2),'Present');
% % % % % % % plot(X,Y,'r')
% % % % % % % [X,Y,T,AUC3] = perfcurve(Y_test,final_scores5(:,3),'Unknown');
% % % % % % % plot(X,Y,'k')
% % % % % % % legend(['Absent AUC: ',num2str(round(AUC1,3))],['Present AUC: ',num2str(round(AUC2,3))],['Unknown AUC: ',num2str(round(AUC3,3))],...
% % % % % % %         'Location','southeast');
% % % % % % % xlabel('False positive rate');
% % % % % % % ylabel('True positive rate');


% % % figure("Position",[524,73,745,889]);
% % % 
% % % subplot(3,2,1)
% % % cm = confusionchart(Y_test,predictedLabels_transformer);
% % % title(['Transformer Accuracy: ',num2str(round(test_acc_transformer,2))]);
% % % subplot(3,2,2)
% % % hold on
% % % box on
% % % grid on
% % % [X,Y,T,AUC1] = perfcurve(Y_test,scores_transformer(:,1),'Abnormal');
% % % plot(X,Y,'r')
% % % [X,Y,T,AUC2] = perfcurve(Y_test,scores_transformer(:,2),'Normal');
% % % plot(X,Y,'g')
% % % legend(['Abnormal AUC: ',num2str(round(AUC1,3))],['Normal AUC: ',num2str(round(AUC2,3))],...
% % %         'Location','southeast');
% % % xlabel('False positive rate');
% % % ylabel('True positive rate');
% % % 
% % % subplot(3,2,3)
% % % cm = confusionchart(Y_test,predictedLabels_NN);
% % % title(['NN Accuracy: ',num2str(round(test_acc_NN,2))]);
% % % subplot(3,2,4)
% % % hold on
% % % box on
% % % grid on
% % % [X,Y,T,AUC1] = perfcurve(Y_test,scores_NN(:,1),'Abnormal');
% % % plot(X,Y,'r')
% % % [X,Y,T,AUC2] = perfcurve(Y_test,scores_NN(:,2),'Normal');
% % % plot(X,Y,'g')
% % % legend(['Abnormal AUC: ',num2str(round(AUC1,3))],['Normal AUC: ',num2str(round(AUC2,3))],...
% % %         'Location','southeast');
% % % xlabel('False positive rate');
% % % ylabel('True positive rate');
% % % 
% % % subplot(3,2,5)
% % % cm = confusionchart(Y_test,voted_array);
% % % title(['Average Voting Accuracy: ',num2str(round(test_acc_final,2))]);
% % % subplot(3,2,6)
% % % hold on
% % % box on
% % % grid on
% % % [X,Y,T,AUC1] = perfcurve(Y_test,final_scores(:,1),'Abnormal');
% % % plot(X,Y,'r')
% % % [X,Y,T,AUC2] = perfcurve(Y_test,final_scores(:,2),'Normal');
% % % plot(X,Y,'g')
% % % legend(['Abnormal AUC: ',num2str(round(AUC1,3))],['Normal AUC: ',num2str(round(AUC2,3))],...
% % %         'Location','southeast');
% % % xlabel('False positive rate');
% % % ylabel('True positive rate');

end

figure("Position",[524,73,745,889]);

overall_p = categorical;
overall_o = categorical;
overall_scores = single.empty;
for p = 1:length(all_predicitons_transformer)
    overall_p = [overall_p;all_predicitons_transformer{p,1}];
    overall_o = [overall_o;all_original{p,1}];
    overall_scores = [overall_scores;all_scores_transformer{p,1}];
end
overall_acc = (sum(categorical(overall_p) == overall_o)./numel(overall_o)).*100;
subplot(3,2,1)
cm = confusionchart(overall_o,overall_p);
title(['Overall Transformer Accuracy: ',num2str(round(overall_acc,2))]);
subplot(3,2,2)
hold on
box on
grid on
[X,Y,T,AUC1] = perfcurve(overall_o,overall_scores(:,1),'Absent');
plot(X,Y,'g')
[X,Y,T,AUC2] = perfcurve(overall_o,overall_scores(:,2),'Present');
plot(X,Y,'r')
[X,Y,T,AUC3] = perfcurve(overall_o,overall_scores(:,3),'Unknown');
plot(X,Y,'k')
legend(['Absent AUC: ',num2str(round(AUC1,3))],['Present AUC: ',num2str(round(AUC2,3))],['Unknown AUC: ',num2str(round(AUC3,3))],...
        'Location','southeast');
xlabel('False positive rate');
ylabel('True positive rate');

overall_p = categorical;
overall_o = categorical;
overall_scores = single.empty;
for p = 1:length(all_predicitons_NN)
    overall_p = [overall_p;all_predicitons_NN{p,1}];
    overall_o = [overall_o;all_original{p,1}];
    overall_scores = [overall_scores;all_scores_NN{p,1}];
end
overall_acc = (sum(categorical(overall_p) == overall_o)./numel(overall_o)).*100;
subplot(3,2,3)
cm = confusionchart(overall_o,overall_p);
title(['Overall NN Accuracy: ',num2str(round(overall_acc,2))]);
subplot(3,2,4)
hold on
box on
grid on
[X,Y,T,AUC1] = perfcurve(overall_o,overall_scores(:,1),'Absent');
plot(X,Y,'g')
[X,Y,T,AUC2] = perfcurve(overall_o,overall_scores(:,2),'Present');
plot(X,Y,'r')
[X,Y,T,AUC3] = perfcurve(overall_o,overall_scores(:,3),'Unknown');
plot(X,Y,'k')
legend(['Absent AUC: ',num2str(round(AUC1,3))],['Present AUC: ',num2str(round(AUC2,3))],['Unknown AUC: ',num2str(round(AUC3,3))],...
        'Location','southeast');
xlabel('False positive rate');
ylabel('True positive rate');

overall_p = categorical;
overall_o = categorical;
overall_scores = single.empty;
for p = 1:length(all_predicitons_voting)
    overall_p = [overall_p;all_predicitons_voting{p,1}];
    overall_o = [overall_o;all_original{p,1}];
    overall_scores = [overall_scores;all_scores_voting{p,1}];
end
overall_acc = (sum(categorical(overall_p) == overall_o)./numel(overall_o)).*100;
subplot(3,2,5)
cm = confusionchart(overall_o,overall_p);
title(['Overall Voting Accuracy: ',num2str(round(overall_acc,2))]);
subplot(3,2,6)
hold on
box on
grid on
[X,Y,T,AUC1] = perfcurve(overall_o,overall_scores(:,1),'Absent');
plot(X,Y,'g')
[X,Y,T,AUC2] = perfcurve(overall_o,overall_scores(:,2),'Present');
plot(X,Y,'r')
[X,Y,T,AUC3] = perfcurve(overall_o,overall_scores(:,3),'Unknown');
plot(X,Y,'k')
legend(['Absent AUC: ',num2str(round(AUC1,3))],['Present AUC: ',num2str(round(AUC2,3))],['Unknown AUC: ',num2str(round(AUC3,3))],...
        'Location','southeast');
xlabel('False positive rate');
ylabel('True positive rate');



% folds_predictions = cell.empty;
% folds_scores = cell.empty;
% for fold_id = 1:10
%     final_predictions = [all_predicitons_transformer{fold_id,1},...
%                          all_predicitons_NN{fold_id,1},...
%                          all_predicitons_LSTM{fold_id,1},...
%                          all_predicitons_voting{fold_id,1}];
%     final_scores = [all_scores_transformer{fold_id,1},...
%                     all_scores_NN{fold_id,1},...
%                     all_scores_LSTM{fold_id,1},...
%                     all_scores_voting{fold_id,1}];
% 
%     final_label = categorical;
%     final_scores2 = zeros(length(final_predictions),3);
%     for test_id = 1:length(final_predictions)
% %         testing_example = final_predictions(test_id,:);
%         testing_score = final_scores(test_id,:);
% 
% %         final_label(test_id,1) = testing_example(1,4);
%         final_scores2(test_id,:) = testing_score(1,10:12);  
% 
%         if testing_example(1,1) == categorical("Unknown")
% %            final_label(test_id,1) = "Unknown";
%            final_scores2(test_id,:) = testing_score(1,1:3); %% Take from Transformer
%         end
% 
%         if testing_example(1,3) == categorical("Present")
% %            final_label(test_id,1) = "Present";
%            final_scores2(test_id,:) = testing_score(1,7:9); %% Take from LSTM
%         end
% 
%     end
% 
% folds_predictions{fold_id,1} = final_label;
% folds_scores{fold_id,1} = final_scores2;
% 
% end
% 
% % VOTING
% for fold_id = 1:10
%     selected_fold = folds_scores{fold_id,1};
% labels_final = categorical(["Absent";"Present";"Unknown"]);
% final_array = categorical;
% for h = 1:size(selected_fold,1)
%     [max_value,max_id] = max(selected_fold(h,:));
%     final_array(h,1) = labels_final(max_id);
% end
% 
% folds_predictions{fold_id,1} = final_array;
% 
% end
% end
% %% VOTING
% for fold_id = 1:10
% selected_transformer = all_scores_transformer{fold_id,1};
% selected_NN = all_scores_NN{fold_id,1};
% selected_LSTM = all_scores_LSTM{fold_id,1};
% 
% % selected_transformer(:,3) = selected_transformer(:,3) .* 2;
% % selected_NN(:,1) = selected_NN(:,1) .* 2;
% % % selected_NN(:,3) = selected_NN(:,3) .* 1/2;
% % selected_LSTM(:,2) = selected_LSTM(:,2) .* 2;
% % % selected_LSTM(:,3) = selected_LSTM(:,3) .* 1/4;
% 
% % VOTING 2
% final_scores = selected_transformer + selected_NN;
% final_scores = final_scores./(sum(final_scores,2));
% labels_final = categorical(["Absent";"Present";"Unknown"]);
% voted_array = categorical;
% for h = 1:size(final_scores,1)
%     [max_value,max_id] = max(final_scores(h,:));
%     voted_array(h,1) = labels_final(max_id);
% end
% folds_predictions{fold_id,1} = voted_array;
% folds_scores{fold_id,1} = final_scores;
% 
% end

% figure("Position",[524,73,745,889]);
% 
% overall_p = categorical;
% overall_o = categorical;
% overall_scores = single.empty;
% for p = 1:length(all_predicitons_transformer)
%     overall_p = [overall_p;all_predicitons_transformer{p,1}];
%     overall_o = [overall_o;all_original{p,1}];
%     overall_scores = [overall_scores;all_scores_transformer{p,1}];
% end
% overall_acc = (sum(categorical(overall_p) == overall_o)./numel(overall_o)).*100;
% subplot(5,2,1)
% cm = confusionchart(overall_o,overall_p);
% title(['Overall Transformer Accuracy: ',num2str(round(overall_acc,2))]);
% subplot(5,2,2)
% hold on
% box on
% grid on
% [X,Y,T,AUC1] = perfcurve(overall_o,overall_scores(:,1),'Absent');
% plot(X,Y,'g')
% [X,Y,T,AUC2] = perfcurve(overall_o,overall_scores(:,2),'Present');
% plot(X,Y,'r')
% [X,Y,T,AUC3] = perfcurve(overall_o,overall_scores(:,3),'Unknown');
% plot(X,Y,'k')
% legend(['Absent AUC: ',num2str(round(AUC1,3))],['Present AUC: ',num2str(round(AUC2,3))],['Unknown AUC: ',num2str(round(AUC3,3))],...
%         'Location','southeast');
% xlabel('False positive rate');
% ylabel('True positive rate');
% 
% overall_p = categorical;
% overall_o = categorical;
% overall_scores = single.empty;
% for p = 1:length(all_predicitons_NN)
%     overall_p = [overall_p;all_predicitons_NN{p,1}];
%     overall_o = [overall_o;all_original{p,1}];
%     overall_scores = [overall_scores;all_scores_NN{p,1}];
% end
% overall_acc = (sum(categorical(overall_p) == overall_o)./numel(overall_o)).*100;
% subplot(5,2,3)
% cm = confusionchart(overall_o,overall_p);
% title(['Overall NN Accuracy: ',num2str(round(overall_acc,2))]);
% subplot(5,2,4)
% hold on
% box on
% grid on
% [X,Y,T,AUC1] = perfcurve(overall_o,overall_scores(:,1),'Absent');
% plot(X,Y,'g')
% [X,Y,T,AUC2] = perfcurve(overall_o,overall_scores(:,2),'Present');
% plot(X,Y,'r')
% [X,Y,T,AUC3] = perfcurve(overall_o,overall_scores(:,3),'Unknown');
% plot(X,Y,'k')
% legend(['Absent AUC: ',num2str(round(AUC1,3))],['Present AUC: ',num2str(round(AUC2,3))],['Unknown AUC: ',num2str(round(AUC3,3))],...
%         'Location','southeast');
% xlabel('False positive rate');
% ylabel('True positive rate');
% 
% overall_p = categorical;
% overall_o = categorical;
% overall_scores = single.empty;
% for p = 1:length(all_predicitons_LSTM)
%     overall_p = [overall_p;all_predicitons_LSTM{p,1}];
%     overall_o = [overall_o;all_original{p,1}];
%     overall_scores = [overall_scores;all_scores_LSTM{p,1}];
% end
% overall_acc = (sum(categorical(overall_p) == overall_o)./numel(overall_o)).*100;
% subplot(5,2,5)
% cm = confusionchart(overall_o,overall_p);
% title(['Overall LSTM Accuracy: ',num2str(round(overall_acc,2))]);
% subplot(5,2,6)
% hold on
% box on
% grid on
% [X,Y,T,AUC1] = perfcurve(overall_o,overall_scores(:,1),'Absent');
% plot(X,Y,'g')
% [X,Y,T,AUC2] = perfcurve(overall_o,overall_scores(:,2),'Present');
% plot(X,Y,'r')
% [X,Y,T,AUC3] = perfcurve(overall_o,overall_scores(:,3),'Unknown');
% plot(X,Y,'k')
% legend(['Absent AUC: ',num2str(round(AUC1,3))],['Present AUC: ',num2str(round(AUC2,3))],['Unknown AUC: ',num2str(round(AUC3,3))],...
%         'Location','southeast');
% xlabel('False positive rate');
% ylabel('True positive rate');
% 
% overall_p = categorical;
% overall_o = categorical;
% overall_scores = single.empty;
% for p = 1:length(all_predicitons_voting)
%     overall_p = [overall_p;all_predicitons_voting{p,1}];
%     overall_o = [overall_o;all_original{p,1}];
%     overall_scores = [overall_scores;all_scores_voting{p,1}];
% end
% overall_acc = (sum(categorical(overall_p) == overall_o)./numel(overall_o)).*100;
% subplot(5,2,7)
% cm = confusionchart(overall_o,overall_p);
% title(['Overall Voting Accuracy: ',num2str(round(overall_acc,2))]);
% subplot(5,2,8)
% hold on
% box on
% grid on
% [X,Y,T,AUC1] = perfcurve(overall_o,overall_scores(:,1),'Absent');
% plot(X,Y,'g')
% [X,Y,T,AUC2] = perfcurve(overall_o,overall_scores(:,2),'Present');
% plot(X,Y,'r')
% [X,Y,T,AUC3] = perfcurve(overall_o,overall_scores(:,3),'Unknown');
% plot(X,Y,'k')
% legend(['Absent AUC: ',num2str(round(AUC1,3))],['Present AUC: ',num2str(round(AUC2,3))],['Unknown AUC: ',num2str(round(AUC3,3))],...
%         'Location','southeast');
% xlabel('False positive rate');
% ylabel('True positive rate');
% 
% overall_p = categorical;
% overall_o = categorical;
% overall_scores = single.empty;
% for p = 1:length(folds_predictions)
%     overall_p = [overall_p;folds_predictions{p,1}];
%     overall_o = [overall_o;all_original{p,1}];
%     overall_scores = [overall_scores;folds_scores{p,1}];
% end
% overall_acc = (sum(categorical(overall_p) == overall_o)./numel(overall_o)).*100;
% subplot(5,2,9)
% cm = confusionchart(overall_o,overall_p);
% title(['Overall Voting (No LSTM) Accuracy: ',num2str(round(overall_acc,2))]);
% subplot(5,2,10)
% hold on
% box on
% grid on
% [X,Y,T,AUC1] = perfcurve(overall_o,overall_scores(:,1),'Absent');
% plot(X,Y,'g')
% [X,Y,T,AUC2] = perfcurve(overall_o,overall_scores(:,2),'Present');
% plot(X,Y,'r')
% [X,Y,T,AUC3] = perfcurve(overall_o,overall_scores(:,3),'Unknown');
% plot(X,Y,'k')
% legend(['Absent AUC: ',num2str(round(AUC1,3))],['Present AUC: ',num2str(round(AUC2,3))],['Unknown AUC: ',num2str(round(AUC3,3))],...
%         'Location','southeast');
% xlabel('False positive rate');
% ylabel('True positive rate');

% overall_p = categorical;
% overall_o = categorical;
% overall_scores = single.empty;
% for p = 1:length(folds_predictions)
%     overall_p = [overall_p;folds_predictions{p,1}];
%     overall_o = [overall_o;all_original{p,1}];
%     overall_scores = [overall_scores;folds_scores{p,1}];
% end
% overall_acc = (sum(categorical(overall_p) == overall_o)./numel(overall_o)).*100;
% subplot(5,2,9)
% cm = confusionchart(overall_o,overall_p);
% title(['Overall Selected Accuracy: ',num2str(round(overall_acc,2))]);
% subplot(5,2,10)
% hold on
% box on
% grid on
% [X,Y,T,AUC1] = perfcurve(overall_o,overall_scores(:,1),'Absent');
% plot(X,Y,'g')
% [X,Y,T,AUC2] = perfcurve(overall_o,overall_scores(:,2),'Present');
% plot(X,Y,'r')
% [X,Y,T,AUC3] = perfcurve(overall_o,overall_scores(:,3),'Unknown');
% plot(X,Y,'k')
% legend(['Absent AUC: ',num2str(round(AUC1,3))],['Present AUC: ',num2str(round(AUC2,3))],['Unknown AUC: ',num2str(round(AUC3,3))],...
%         'Location','southeast');
% xlabel('False positive rate');
% ylabel('True positive rate');








% 
% % LSTM
% [net_LSTM,predictedLabels_LSTM,scores_LSTM] = train_LSTM(transformer_features_train_edited,Y_train_edited,transformer_features_test,Y_test);
% test_acc_LSTM = sum(predictedLabels_LSTM == Y_test)/numel(Y_test)*100;
% all_predicitons_LSTM{i,1} = predictedLabels_LSTM;
% all_scores_LSTM{i,1} = scores_LSTM;
% % figure;confusionchart(Y_test,predictedLabels_LSTM);

% % Random forest
% [net_RF,predictedLabels_RF,scores_RF] = train_RF(transformer_features_train_edited,Y_train_edited,transformer_features_test,Y_test);
% test_acc_RF = (sum(categorical(predictedLabels_RF) == Y_test)./numel(Y_test)).*100;
% % figure;confusionchart(Y_test,predictedLabels_RF);
% all_predicitons_RF{i,1} = predictedLabels_RF;
% all_scores_RF{i,1} = scores_RF;
% 
% % SVM
% [net_SVM,predictedLabels_SVM,scores_SVM] = train_SVM(transformer_features_train_edited,Y_train_edited,transformer_features_test,Y_test);
% test_acc_SVM = (sum(categorical(predictedLabels_SVM) == Y_test)./numel(Y_test)).*100;
% % figure;confusionchart(Y_test,predictedLabels_SVM);
% all_predicitons_SVM{i,1} = predictedLabels_SVM;
% all_scores_SVM{i,1} = scores_SVM;
% 
% % ADABOOST
% [net_ADA,predictedLabels_ADA,scores_ADA] = train_ADA(transformer_features_train_edited,Y_train_edited,transformer_features_test,Y_test);
% test_acc_ADA = (sum(categorical(predictedLabels_ADA) == Y_test)./numel(Y_test)).*100;
% % figure;confusionchart(Y_test,predictedLabels_ADA);
% all_predicitons_ADA{i,1} = predictedLabels_ADA;
% all_scores_ADA{i,1} = scores_ADA;
% 
% % kNN
% [net_kNN,predictedLabels_kNN,scores_kNN] = train_kNN(transformer_features_train_edited,Y_train_edited,transformer_features_test,Y_test);
% test_acc_kNN = (sum(categorical(predictedLabels_kNN) == Y_test)./numel(Y_test)).*100;
% % figure;confusionchart(Y_test,predictedLabels_kNN);
% all_predicitons_kNN{i,1} = predictedLabels_kNN;
% all_scores_kNN{i,1} = scores_kNN;
% 
% % Basic neural net
% [net_neural,predictedLabels_neural,scores_neural] = train_neural(transformer_features_train_edited,Y_train_edited,transformer_features_test,Y_test);
% test_acc_neural = (sum(categorical(predictedLabels_neural) == Y_test)./numel(Y_test)).*100;
% % figure;confusionchart(Y_test,predictedLabels_neural);
% all_predicitons_neural{i,1} = predictedLabels_neural;
% all_scores_neural{i,1} = scores_neural;
% 
% % Naive Bayes
% [net_nb,predictedLabels_nb,scores_nb] = train_nb(transformer_features_train_edited,Y_train_edited,transformer_features_test,Y_test);
% test_acc_nb = (sum(categorical(predictedLabels_nb) == Y_test)./numel(Y_test)).*100;
% % figure;confusionchart(Y_test,predictedLabels_nb);
% all_predicitons_nb{i,1} = predictedLabels_nb;
% all_scores_nb{i,1} = scores_nb;
% 
% % discriminant analysis
% [net_discr,predictedLabels_discr,scores_discr] = train_discr(transformer_features_train_edited,Y_train_edited,transformer_features_test,Y_test);
% test_acc_discr = (sum(categorical(predictedLabels_discr) == Y_test)./numel(Y_test)).*100;
% % figure;confusionchart(Y_test,predictedLabels_discr);
% all_predicitons_discr{i,1} = predictedLabels_discr;
% all_scores_discr{i,1} = scores_discr;




% % Picking most repeated label
% prediction_pick = mode([predictedLabels_transformer,...
%                         predictedLabels_NN,...
%                         predictedLabels_LSTM],2);
% test_acc_pick = (sum(categorical(prediction_pick) == Y_test)./numel(Y_test)).*100;

% figure;confusionchart(Y_test,prediction_pick);
%                         predictedLabels_RF,...
%                         predictedLabels_SVM,...
%                         predictedLabels_ADA,...
%                         predictedLabels_kNN,...
%                         predictedLabels_neural,...
%                         predictedLabels_nb,...
%                         predictedLabels_discr,...


% % Trained voting
% scores_vector_train = [[scores_transformer_train;scores_NN_train(length(scores_transformer_train)+1:end,:)],scores_NN_train,0.5.*scores_RF_train,0.5.*scores_SVM_train,0.5.*scores_ADA_train,...
%                        0.5.*scores_kNN_train,0.5.*scores_neural_train,0.5.*scores_nb_train,0.5.*scores_discr_train];
% scores_vector_test = [scores_transformer,scores_NN,0.5.*scores_RF,0.5.*scores_SVM,0.5.*scores_ADA,...
%                       0.5.*scores_kNN,0.5.*scores_neural,0.5.*scores_nb,0.5.*scores_discr];
% numFeatures2 = size(scores_vector_train,2);
% 
% layers2 = [
%     featureInputLayer(numFeatures2,'Name','input',Normalization="none")
%     fullyConnectedLayer(3, 'Name','fc')
%     softmaxLayer('Name','sm')
%     classificationLayer
%     ];
% 
% options = trainingOptions('adam', ...
%     'ExecutionEnvironment','gpu', ...
%     'MaxEpochs',7000, ...
%     'MiniBatchSize',32, ...
%     'InitialLearnRate',0.00001, ...
%     'LearnRateSchedule',"piecewise", ...
%     'LearnRateDropFactor',0.1, ...
%     'LearnRateDropPeriod',6500, ...
%     'L2Regularization',0.00001, ...
%     'ValidationData',{scores_vector_test,Y_test}, ...
%     'ValidationFrequency',100, ...
%     'Shuffle','every-epoch', ...
%     'Verbose',false,...
%      'Plots','training-progress');
% % ,...
% %      'Plots','training-progress'
% rng('default')
% clear net_scores
% [net_scores info] = trainNetwork(scores_vector_train,Y_train_edited,layers2,options);
% [prediction_scores,scores_scores] = classify(net_scores,scores_vector_test,'MiniBatchSize',32);
% test_acc_trained_voting = sum(prediction_scores == Y_test)/numel(Y_test)*100;
% all_predicitons_trained_voting{i,1} = prediction_scores;
% all_scores_trained_voting{i,1} = scores_scores;
% % figure;confusionchart(Y_test,prediction_scores);

%%














% % % figure("Position",[1,41,1920,963]);
% % % 
% % % subplot(4,6,1)
% % % cm = confusionchart(Y_test,predictedLabels_transformer);
% % % title(['Transformer Accuracy: ',num2str(round(test_acc_transformer,2))]);
% % % subplot(4,6,2)
% % % hold on
% % % box on
% % % grid on
% % % [X,Y,T,AUC1] = perfcurve(Y_test,scores_transformer(:,1),'Absent');
% % % plot(X,Y,'g')
% % % [X,Y,T,AUC2] = perfcurve(Y_test,scores_transformer(:,2),'Present');
% % % plot(X,Y,'r')
% % % [X,Y,T,AUC3] = perfcurve(Y_test,scores_transformer(:,3),'Unknown');
% % % plot(X,Y,'k')
% % % legend(['Absent AUC: ',num2str(round(AUC1,3))],['Present AUC: ',num2str(round(AUC2,3))],['Unknown AUC: ',num2str(round(AUC3,3))],...
% % %         'Location','southeast');
% % % xlabel('False positive rate');
% % % ylabel('True positive rate');
% % % 
% % % subplot(4,6,3)
% % % cm = confusionchart(Y_test,predictedLabels_NN);
% % % title(['NN Accuracy: ',num2str(round(test_acc_NN,2))]);
% % % subplot(4,6,4)
% % % hold on
% % % box on
% % % grid on
% % % [X,Y,T,AUC1] = perfcurve(Y_test,scores_NN(:,1),'Absent');
% % % plot(X,Y,'g')
% % % [X,Y,T,AUC2] = perfcurve(Y_test,scores_NN(:,2),'Present');
% % % plot(X,Y,'r')
% % % [X,Y,T,AUC3] = perfcurve(Y_test,scores_NN(:,3),'Unknown');
% % % plot(X,Y,'k')
% % % legend(['Absent AUC: ',num2str(round(AUC1,3))],['Present AUC: ',num2str(round(AUC2,3))],['Unknown AUC: ',num2str(round(AUC3,3))],...
% % %         'Location','southeast');
% % % xlabel('False positive rate');
% % % ylabel('True positive rate');
% % % 
% % % subplot(4,6,5)
% % % cm = confusionchart(Y_test,predictedLabels_RF);
% % % title(['RF Accuracy: ',num2str(round(test_acc_RF,2))]);
% % % subplot(4,6,6)
% % % hold on
% % % box on
% % % grid on
% % % [X,Y,T,AUC1] = perfcurve(Y_test,scores_RF(:,1),'Absent');
% % % plot(X,Y,'g')
% % % [X,Y,T,AUC2] = perfcurve(Y_test,scores_RF(:,2),'Present');
% % % plot(X,Y,'r')
% % % [X,Y,T,AUC3] = perfcurve(Y_test,scores_RF(:,3),'Unknown');
% % % plot(X,Y,'k')
% % % legend(['Absent AUC: ',num2str(round(AUC1,3))],['Present AUC: ',num2str(round(AUC2,3))],['Unknown AUC: ',num2str(round(AUC3,3))],...
% % %         'Location','southeast');
% % % xlabel('False positive rate');
% % % ylabel('True positive rate');
% % % 
% % % subplot(4,6,7)
% % % cm = confusionchart(Y_test,predictedLabels_SVM);
% % % title(['SVM Accuracy: ',num2str(round(test_acc_SVM,2))]);
% % % subplot(4,6,8)
% % % hold on
% % % box on
% % % grid on
% % % [X,Y,T,AUC1] = perfcurve(Y_test,scores_SVM(:,1),'Absent');
% % % plot(X,Y,'g')
% % % [X,Y,T,AUC2] = perfcurve(Y_test,scores_SVM(:,2),'Present');
% % % plot(X,Y,'r')
% % % [X,Y,T,AUC3] = perfcurve(Y_test,scores_SVM(:,3),'Unknown');
% % % plot(X,Y,'k')
% % % legend(['Absent AUC: ',num2str(round(AUC1,3))],['Present AUC: ',num2str(round(AUC2,3))],['Unknown AUC: ',num2str(round(AUC3,3))],...
% % %         'Location','southeast');
% % % xlabel('False positive rate');
% % % ylabel('True positive rate');
% % % 
% % % subplot(4,6,9)
% % % cm = confusionchart(Y_test,predictedLabels_ADA);
% % % title(['ADA Accuracy: ',num2str(round(test_acc_ADA,2))]);
% % % subplot(4,6,10)
% % % hold on
% % % box on
% % % grid on
% % % [X,Y,T,AUC1] = perfcurve(Y_test,scores_ADA(:,1),'Absent');
% % % plot(X,Y,'g')
% % % [X,Y,T,AUC2] = perfcurve(Y_test,scores_ADA(:,2),'Present');
% % % plot(X,Y,'r')
% % % [X,Y,T,AUC3] = perfcurve(Y_test,scores_ADA(:,3),'Unknown');
% % % plot(X,Y,'k')
% % % legend(['Absent AUC: ',num2str(round(AUC1,3))],['Present AUC: ',num2str(round(AUC2,3))],['Unknown AUC: ',num2str(round(AUC3,3))],...
% % %         'Location','southeast');
% % % xlabel('False positive rate');
% % % ylabel('True positive rate');
% % % 
% % % subplot(4,6,11)
% % % cm = confusionchart(Y_test,predictedLabels_kNN);
% % % title(['kNN Accuracy: ',num2str(round(test_acc_kNN,2))]);
% % % subplot(4,6,12)
% % % hold on
% % % box on
% % % grid on
% % % [X,Y,T,AUC1] = perfcurve(Y_test,scores_kNN(:,1),'Absent');
% % % plot(X,Y,'g')
% % % [X,Y,T,AUC2] = perfcurve(Y_test,scores_kNN(:,2),'Present');
% % % plot(X,Y,'r')
% % % [X,Y,T,AUC3] = perfcurve(Y_test,scores_kNN(:,3),'Unknown');
% % % plot(X,Y,'k')
% % % legend(['Absent AUC: ',num2str(round(AUC1,3))],['Present AUC: ',num2str(round(AUC2,3))],['Unknown AUC: ',num2str(round(AUC3,3))],...
% % %         'Location','southeast');
% % % xlabel('False positive rate');
% % % ylabel('True positive rate');
% % % 
% % % subplot(4,6,13)
% % % cm = confusionchart(Y_test,predictedLabels_neural);
% % % title(['Basic Nueral Accuracy: ',num2str(round(test_acc_neural,2))]);
% % % subplot(4,6,14)
% % % hold on
% % % box on
% % % grid on
% % % [X,Y,T,AUC1] = perfcurve(Y_test,scores_neural(:,1),'Absent');
% % % plot(X,Y,'g')
% % % [X,Y,T,AUC2] = perfcurve(Y_test,scores_neural(:,2),'Present');
% % % plot(X,Y,'r')
% % % [X,Y,T,AUC3] = perfcurve(Y_test,scores_neural(:,3),'Unknown');
% % % plot(X,Y,'k')
% % % legend(['Absent AUC: ',num2str(round(AUC1,3))],['Present AUC: ',num2str(round(AUC2,3))],['Unknown AUC: ',num2str(round(AUC3,3))],...
% % %         'Location','southeast');
% % % xlabel('False positive rate');
% % % ylabel('True positive rate');
% % % 
% % % subplot(4,6,15)
% % % cm = confusionchart(Y_test,predictedLabels_nb);
% % % title(['Naive Bayes Accuracy: ',num2str(round(test_acc_nb,2))]);
% % % subplot(4,6,16)
% % % hold on
% % % box on
% % % grid on
% % % [X,Y,T,AUC1] = perfcurve(Y_test,scores_nb(:,1),'Absent');
% % % plot(X,Y,'g')
% % % [X,Y,T,AUC2] = perfcurve(Y_test,scores_nb(:,2),'Present');
% % % plot(X,Y,'r')
% % % [X,Y,T,AUC3] = perfcurve(Y_test,scores_nb(:,3),'Unknown');
% % % plot(X,Y,'k')
% % % legend(['Absent AUC: ',num2str(round(AUC1,3))],['Present AUC: ',num2str(round(AUC2,3))],['Unknown AUC: ',num2str(round(AUC3,3))],...
% % %         'Location','southeast');
% % % xlabel('False positive rate');
% % % ylabel('True positive rate');
% % % 
% % % subplot(4,6,17)
% % % cm = confusionchart(Y_test,predictedLabels_discr);
% % % title(['Discriminant Accuracy: ',num2str(round(test_acc_discr,2))]);
% % % subplot(4,6,18)
% % % hold on
% % % box on
% % % grid on
% % % [X,Y,T,AUC1] = perfcurve(Y_test,scores_discr(:,1),'Absent');
% % % plot(X,Y,'g')
% % % [X,Y,T,AUC2] = perfcurve(Y_test,scores_discr(:,2),'Present');
% % % plot(X,Y,'r')
% % % [X,Y,T,AUC3] = perfcurve(Y_test,scores_discr(:,3),'Unknown');
% % % plot(X,Y,'k')
% % % legend(['Absent AUC: ',num2str(round(AUC1,3))],['Present AUC: ',num2str(round(AUC2,3))],['Unknown AUC: ',num2str(round(AUC3,3))],...
% % %         'Location','southeast');
% % % xlabel('False positive rate');
% % % ylabel('True positive rate');
% % % 
% % % subplot(4,6,19)
% % % cm = confusionchart(Y_test,predictedLabels_LSTM);
% % % title(['LSTM Accuracy: ',num2str(round(test_acc_LSTM,2))]);
% % % subplot(4,6,20)
% % % hold on
% % % box on
% % % grid on
% % % [X,Y,T,AUC1] = perfcurve(Y_test,scores_LSTM(:,1),'Absent');
% % % plot(X,Y,'g')
% % % [X,Y,T,AUC2] = perfcurve(Y_test,scores_LSTM(:,2),'Present');
% % % plot(X,Y,'r')
% % % [X,Y,T,AUC3] = perfcurve(Y_test,scores_LSTM(:,3),'Unknown');
% % % plot(X,Y,'k')
% % % legend(['Absent AUC: ',num2str(round(AUC1,3))],['Present AUC: ',num2str(round(AUC2,3))],['Unknown AUC: ',num2str(round(AUC3,3))],...
% % %         'Location','southeast');
% % % xlabel('False positive rate');
% % % ylabel('True positive rate');
% % % 
% % % subplot(4,6,21)
% % % cm = confusionchart(Y_test,prediction_pick);
% % % title(['Most Predicted Accuracy: ',num2str(round(test_acc_pick,2))]);
% % % subplot(4,6,22)
% % % hold on
% % % box on
% % % grid on
% % % [X,Y,T,AUC1] = perfcurve(Y_test,final_scores(:,1),'Absent');
% % % plot(X,Y,'g')
% % % [X,Y,T,AUC2] = perfcurve(Y_test,final_scores(:,2),'Present');
% % % plot(X,Y,'r')
% % % [X,Y,T,AUC3] = perfcurve(Y_test,final_scores(:,3),'Unknown');
% % % plot(X,Y,'k')
% % % legend(['Absent AUC: ',num2str(round(AUC1,3))],['Present AUC: ',num2str(round(AUC2,3))],['Unknown AUC: ',num2str(round(AUC3,3))],...
% % %         'Location','southeast');
% % % xlabel('False positive rate');
% % % ylabel('True positive rate');
% % % 
% % % 
% % % subplot(4,6,23)
% % % cm = confusionchart(Y_test,voted_array);
% % % title(['Average Voting Accuracy: ',num2str(round(test_acc_final,2))]);
% % % subplot(4,6,24)
% % % hold on
% % % box on
% % % grid on
% % % [X,Y,T,AUC1] = perfcurve(Y_test,final_scores(:,1),'Absent');
% % % plot(X,Y,'g')
% % % [X,Y,T,AUC2] = perfcurve(Y_test,final_scores(:,2),'Present');
% % % plot(X,Y,'r')
% % % [X,Y,T,AUC3] = perfcurve(Y_test,final_scores(:,3),'Unknown');
% % % plot(X,Y,'k')
% % % legend(['Absent AUC: ',num2str(round(AUC1,3))],['Present AUC: ',num2str(round(AUC2,3))],['Unknown AUC: ',num2str(round(AUC3,3))],...
% % %         'Location','southeast');
% % % xlabel('False positive rate');
% % % ylabel('True positive rate');

%%



% overall_p = categorical;
% overall_o = categorical;
% for p = 1:length(all_predicitons)
%     overall_p = [overall_p;all_predicitons{p,1}];
%     overall_o = [overall_o;all_original{p,1}];
% end
% figure;
% cm = confusionchart(overall_o,overall_p);
% 
% overall_acc = (sum(categorical(overall_p) == overall_o)./numel(overall_o)).*100





% 
% all_accuracies(i,2) = test_acc2;
% 
% all_accuracies(i,:)

% end





















features = [train_demog,train_handcrafted];
featurestest = [test_demog,test_handcrafted];


numFeatures2 = size(features,2);
layers2 = [
    featureInputLayer(numFeatures2,'Name','input',Normalization="zscore")
    fullyConnectedLayer(32, 'Name','fc')
    fullyConnectedLayer(3, 'Name','fc')
    softmaxLayer('Name','sm')
    classificationLayer('Classes',unique(Y_train),'ClassWeights',weights)
    ];

options = trainingOptions('adam', ...
    'ExecutionEnvironment','gpu', ...
    'MaxEpochs',300, ...
    'MiniBatchSize',miniBatchSize, ...
    'InitialLearnRate',0.001, ...
    'LearnRateSchedule',"piecewise", ...
    'LearnRateDropFactor',0.1, ...
    'LearnRateDropPeriod',250, ...
    'L2Regularization',0.0001, ...
    'ValidationData',{featurestest,Y_test}, ...
    'ValidationFrequency',100, ...
    'Shuffle','every-epoch', ...
    'Verbose',false,...
     'Plots','training-progress');

rng('default')
clear net2
[net2 info] = trainNetwork(features,Y_train,layers2,options);
[predictionNN,scoresNN] = classify(net2,featurestest,'MiniBatchSize',32);
Accuracy = sum(predictionNN == Y_test)/numel(Y_test)*100;
% cm = confusionchart(Y_test,predictionNN);





Mdl = fitcensemble(features,Y_train,'Method','AdaBoostM2','NumLearningCycles',50,'Prior',weights);
[predictionada,scoresada] = predict(Mdl,featurestest);
scoresada = scoresada./(sum(scoresada,2));
predictionada = categorical(string(predictionada));
Accuracy = sum(predictionada == Y_test)/numel(Y_test)*100;
cm = confusionchart(Y_test,predictionada);





model_murmur = TreeBagger(100,features,Y_train,MinLeafSize=7,OOBPrediction="on",Surrogate="off",...
                          OOBPredictorImportance="on",Prior=weights);
[predictionrf,scoresrf] = predict(model_murmur,featurestest);
predictionrf2 = categorical(string(predictionrf));
Accuracy = sum(predictionrf2 == Y_test)/numel(Y_test)*100;
cm = confusionchart(Y_test,predictionrf2);


% figure;
% plot(oobError(model_murmur))
% xlabel("Number of Grown Trees")
% ylabel("Out-of-Bag Classification Error")
% 
% impCART = model_murmur.OOBPermutedPredictorDeltaError;
% 
% figure;
% bar(impCART)
% title("Standard CART")
% ylabel("Predictor Importance Estimates")
% xlabel("Predictors")


rng;
clear Mdl
t = templateSVM('KernelFunction','rbf','BoxConstraint',1,'KernelScale','auto','Standardize',false);
Mdl = fitcecoc(features,Y_train,'Learners',t,'ScoreTransform','none','FitPosterior',0,'Prior',weights);    
[predictionsvm,scoressvm] = predict(Mdl,featurestest);
scoressvm = 1 + scoressvm;
scoressvm = scoressvm./(sum(scoressvm,2));
predictionsvm2 = categorical(string(predictionsvm));
Accuracy = sum(predictionsvm2 == Y_test)/numel(Y_test)*100;
cm = confusionchart(Y_test,predictionrf2);



scores_transformer = scores_transformer;
scores_svm = scoressvm;

final_scores = 1.*scores_transformer + 1.*scores2;
% 0.5.*scores_svm + 0.5.*scoresrf + 0.5.*scoresada + 0.5.*scoresNN;
final_scores = final_scores / 2;

labels_final = categorical(["Absent";"Present";"Unknown"]);
voted_array = categorical;
for h = 1:94
    [max_value,max_id] = max(final_scores(h,:));
    voted_array(h,1) = labels_final(max_id);
end

figure;
subplot(1,2,1)
confusionchart(Y_test,predictedLabels);
subplot(1,2,2)
confusionchart(Y_test,voted_array);


final_scores = 1.*scores_transformer + 1.*scores_NN + 0.5.*scores_RF + 0.5.*scores_SVM + 0.5.*scores_ADA;

features2 = [scores_transformer,scores_NN,0.5.*scores_RF,0.5.*scores_SVM,0.5.*scores_ADA];
numFeatures2 = size(features2,2);
layers2 = [
    featureInputLayer(numFeatures2,'Name','input',Normalization="none")
    fullyConnectedLayer(3, 'Name','fc')
    softmaxLayer('Name','sm')
    classificationLayer
    ];

options = trainingOptions('adam', ...
    'ExecutionEnvironment','gpu', ...
    'MaxEpochs',5000, ...
    'MiniBatchSize',miniBatchSize, ...
    'InitialLearnRate',0.01, ...
    'LearnRateSchedule',"piecewise", ...
    'LearnRateDropFactor',0.1, ...
    'LearnRateDropPeriod',4500, ...
    'L2Regularization',0.0001, ...
    'ValidationData',{features2,Y_test}, ...
    'ValidationFrequency',100, ...
    'Shuffle','every-epoch', ...
    'Verbose',false,...
     'Plots','training-progress');

rng('default')
clear net3
[net3 info] = trainNetwork(features2,Y_test,layers2,options);
[prediction2,scores2] = classify(net3,features2,'MiniBatchSize',32);
Accuracy = sum(prediction2 == Y_test)/numel(Y_test)*100;
% cm = confusionchart(Y_test,predictionNN2);

figure;
subplot(1,2,1)
confusionchart(Y_test,predictedLabels_transformer);
subplot(1,2,2)
confusionchart(Y_test,prediction2);

weights_NN = net3.Layers(2, 1).Weights;
figure;
imagesc(weights_NN)











% % % labels = categorical(["Present","Absent","Unknown"]);
% % % Y_train_edit = Y_train;
% % % for n = 1:length(predictedLabels)
% % %     prediction = predictedLabels(n,1);
% % %     score = scores(n,:);
% % %     [high_value, high_location] = find(score >= 0.7);
% % %     if isempty(high_location) == 0
% % %        Y_train_edit(n,1) = labels(high_location);
% % %     end
% % % end

% % % %%%%%%%%% Augmentation
% % % classes_aug = unique(Y_train);
% % % Augmented_dataset = cell.empty;
% % % Augmented_labels = double.empty;
% % % NumAugmentations = 3000;
% % % for class_count = 1:length(classes_aug)
% % %     
% % % selected_class = classes_aug(class_count,1);
% % % class_idx = find(Y_train == selected_class);
% % % class_data = train_signals(class_idx,1);
% % % labels_of_class = Y_train(class_idx); 
% % % 
% % %     if length(class_data) <= NumAugmentations
% % %   
% % %     augmenter = audioDataAugmenter( ...
% % %     "AugmentationMode","sequential", ...
% % %     "NumAugmentations",floor(NumAugmentations/length(class_data)), ...
% % %     ...
% % %     "TimeStretchProbability",0, ...
% % %     "SpeedupFactorRange", [1,1], ...
% % %     "SpeedupFactor", 0,...
% % %     ...
% % %     "PitchShiftProbability",0, ...
% % %     ...
% % %     "VolumeControlProbability",1, ...
% % %     "VolumeGainRange",[-10,10], ...
% % %     ...
% % %     "AddNoiseProbability",0, ...
% % %     ...
% % %     "TimeShiftProbability",1, ...
% % %     "TimeShiftRange", [-10,10],...
% % %     "TimeShift",10);
% % % 
% % % all_data = cell.empty;
% % % all_gender = double.empty;
% % % all_age = double.empty;
% % % for data_order = 1:length(class_data)
% % % data = augment(augmenter,class_data{data_order,1}',fs);
% % %     for p = 1:length(data.Audio)
% % %         data.Audio{p,1} = data.Audio{p,1}';
% % %     end
% % % all_data = [all_data;class_data{data_order,1};data.Audio];
% % % % [class_count data_order length(class_data)]
% % % end   
% % % 
% % % aug_labels = repmat(labels_of_class(1,1),length(all_data),1);
% % % 
% % % Augmented_dataset = [Augmented_dataset;all_data];
% % % Augmented_labels = [Augmented_labels;aug_labels];
% % % 
% % %     else
% % %     
% % % aug_labels = repmat(categorical(selected_class),length(class_data),1);
% % % 
% % % Augmented_dataset = [Augmented_dataset;class_data];
% % % Augmented_labels = [Augmented_labels;aug_labels];
% % % 
% % %     end
% % %    
% % % end
% % % 
% % % % Randomizing
% % % randomizing2 = randperm(length(Augmented_dataset));
% % % Augmented_dataset = Augmented_dataset(randomizing2,1);
% % % Augmented_labels = Augmented_labels(randomizing2);
% % % for q = 1:length(Augmented_dataset)
% % %     Augmented_dataset{q,1} = normalize(Augmented_dataset{q,1},2);
% % % end



all_predicitons2(i,1) = predictedLabels2;
all_original2(i,1) = Y_test;
all_scores2(i,:) = scores2;
all_accuracies2(i,1) = test_acc2;

array2 = [string(all_accuracies2) string(all_original2) string(all_predicitons2) string(all_scores2)];

[array(i,:);array2(i,:)]



model_outcome = TreeBagger(300,features,labels_outcome);

save_model(model_murmur,classes_murmur,model_outcome,classes_outcome,output_directory);

disp('Done.')

end

function save_model(model_murmur,classes_murmur,model_outcome,classes_outcome,output_directory) %save_PCG_model
% Save results.
filename = fullfile(output_directory,'model.mat');
save(filename,'model_murmur','classes_murmur','model_outcome','classes_outcome','-v7.3');

disp('Done.')
end

function class=get_class_murmur(input_header)

current_header=get_header(input_header);

class=current_header(startsWith(current_header,'#Murmur'));
class=strsplit(class{1},':');
class=strtrim(class{2});

end

function class=get_class_outcome(input_header)

current_header=get_header(input_header);

class=current_header(startsWith(current_header,'#Outcome'));
class=strsplit(class{1},':');
class=strtrim(class{2});

end

function current_header=get_header(input_header)

current_header=fileread(input_header);
current_header=strsplit(current_header,'\n');

end

function current_recordings=load_recordings(input_directory,current_header)

recording_files=get_recording_files(current_header);

current_recordings={};

for j=1:length(recording_files)

    current_recordings{j}=audioread(fullfile(input_directory,strtrim(recording_files{j})));

end

end

function recording_files=get_recording_files(current_header)

recording_files={};

num_locations=strsplit(current_header{1},' ');
num_locations=str2double(num_locations{2});

for j=2:num_locations+1

    current_line=strsplit(current_header{j},' ');
    recording_files{j-1}=current_line{3};

end

end
